/* Copyright 2024 The OpenXLA Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/

/*
  This file is generated using xla/tests/generate_complex_unary_op_samples.py.
  Do not edit!
 */

#include <cmath>
#include <complex>
#include <limits>
#include <tuple>
#include <vector>

#ifndef XLA_TESTS_COMPLEX_UNARY_OP_SAMPLES_H_
#define XLA_TESTS_COMPLEX_UNARY_OP_SAMPLES_H_

namespace complex_unary_op_samples {
// NOLINTBEGIN(whitespace/line_length)

template <class>
constexpr bool dependent_false = false;

template <typename T, int default_dps_deficiency = 0>
struct Log1p {
  typedef std::complex<T> InputType;
  typedef std::complex<T> OutputType;
  typedef T FloatType;
  using TableType = std::vector<std::tuple<InputType, OutputType, FloatType>>;
  static constexpr int dps_deficiency = default_dps_deficiency;
  const TableType get() {
    if constexpr (std::is_same_v<T, float>) {
      const T pi = 3.1415927f;
      const T pi_4 = 0.7853982f;
      const T pi_2 = 1.5707964f;
      const T pi3_4 = 2.3561945f;
      const T zero = 0.0f;
      const T inf = std::numeric_limits<T>::infinity();
      const T min = std::numeric_limits<T>::min();
      const T max = std::numeric_limits<T>::max();
      const TableType table{
          // clang-format off
          // Ignore max 80 character line width style requirement for
          // (i) the readability
          // (ii) the consistency with the local conventions
        /* 0 */ { { -inf, -inf }, { inf, -pi3_4 }, 1.e+00f },
        /* 1 */ { { -max, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 2 */ { { -6.1409603e+25f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 3 */ { { -1.1082383e+13f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 4 */ { { -2.e+00f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 5 */ { { -3.6093321e-13f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 6 */ { { -6.5136393e-26f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 7 */ { { -min, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 8 */ { { zero, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 9 */ { { min, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 10 */ { { 6.5136393e-26f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 11 */ { { 3.6093321e-13f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 12 */ { { 2.e+00f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 13 */ { { 1.1082383e+13f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 14 */ { { 6.1409603e+25f, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 15 */ { { max, -inf }, { inf, -pi_2 }, 1.e+00f },
        /* 16 */ { { inf, -inf }, { inf, -pi_4 }, 1.e+00f },
        /* 17 */ { { -inf, -max }, { inf, -pi }, 1.e+00f },
        /* 18 */ { { -max, -max }, { 8.9069412e+01f, -pi3_4 }, 7.8125e-03f },
        /* 19 */ { { -6.1409603e+25f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 20 */ { { -1.1082383e+13f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 21 */ { { -2.e+00f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 22 */ { { -3.6093321e-13f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 23 */ { { -6.5136393e-26f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 24 */ { { -min, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 25 */ { { zero, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 26 */ { { min, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 27 */ { { 6.5136393e-26f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 28 */ { { 3.6093321e-13f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 29 */ { { 2.e+00f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 30 */ { { 1.1082383e+13f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 31 */ { { 6.1409603e+25f, -max }, { 8.8722839e+01f, -pi_2 }, 7.8125e-03f },
        /* 32 */ { { max, -max }, { 8.9069412e+01f, -pi_4 }, 7.8125e-03f },
        /* 33 */ { { inf, -max }, { inf, zero }, 1.e+00f },
        /* 34 */ { { -inf, -6.1409603e+25f }, { inf, -pi }, 1.e+00f },
        /* 35 */ { { -max, -6.1409603e+25f }, { 8.8722839e+01f, -pi }, 7.8125e-03f },
        /* 36 */ { { -6.1409603e+25f, -6.1409603e+25f }, { 5.9726181e+01f, -pi3_4 }, 1.5625e-02f },
        /* 37 */ { { -1.1082383e+13f, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 38 */ { { -2.e+00f, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 39 */ { { -3.6093321e-13f, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 40 */ { { -6.5136393e-26f, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 41 */ { { -min, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 42 */ { { zero, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 43 */ { { min, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 44 */ { { 6.5136393e-26f, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 45 */ { { 3.6093321e-13f, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 46 */ { { 2.e+00f, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 47 */ { { 1.1082383e+13f, -6.1409603e+25f }, { 5.9379608e+01f, -pi_2 }, 1.5625e-02f },
        /* 48 */ { { 6.1409603e+25f, -6.1409603e+25f }, { 5.9726181e+01f, -pi_4 }, 1.5625e-02f },
        /* 49 */ { { max, -6.1409603e+25f }, { 8.8722839e+01f, -1.8046662e-13f }, 7.8125e-03f },
        /* 50 */ { { inf, -6.1409603e+25f }, { inf, zero }, 1.e+00f },
        /* 51 */ { { -inf, -1.1082383e+13f }, { inf, -pi }, 1.e+00f },
        /* 52 */ { { -max, -1.1082383e+13f }, { 8.8722839e+01f, -pi }, 7.8125e-03f },
        /* 53 */ { { -6.1409603e+25f, -1.1082383e+13f }, { 5.9379608e+01f, -pi }, 1.5625e-02f },
        /* 54 */ { { -1.1082383e+13f, -1.1082383e+13f }, { 3.0382952e+01f, -pi3_4 }, 3.125e-02f },
        /* 55 */ { { -2.e+00f, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 56 */ { { -3.6093321e-13f, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 57 */ { { -6.5136393e-26f, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 58 */ { { -min, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 59 */ { { zero, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 60 */ { { min, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 61 */ { { 6.5136393e-26f, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 62 */ { { 3.6093321e-13f, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 63 */ { { 2.e+00f, -1.1082383e+13f }, { 3.0036377e+01f, -pi_2 }, 3.125e-02f },
        /* 64 */ { { 1.1082383e+13f, -1.1082383e+13f }, { 3.0382952e+01f, -pi_4 }, 3.125e-02f },
        /* 65 */ { { 6.1409603e+25f, -1.1082383e+13f }, { 5.9379608e+01f, -1.8046662e-13f }, 1.5625e-02f },
        /* 66 */ { { max, -1.1082383e+13f }, { 8.8722839e+01f, -3.25682e-26f }, 7.8125e-03f },
        /* 67 */ { { inf, -1.1082383e+13f }, { inf, zero }, 1.e+00f },
        /* 68 */ { { -inf, -2.e+00f }, { inf, -pi }, 1.e+00f },
        /* 69 */ { { -max, -2.e+00f }, { 8.8722839e+01f, -pi }, 7.8125e-03f },
        /* 70 */ { { -6.1409603e+25f, -2.e+00f }, { 5.9379608e+01f, -pi }, 1.5625e-02f },
        /* 71 */ { { -1.1082383e+13f, -2.e+00f }, { 3.0036377e+01f, -pi }, 3.125e-02f },
        /* 72 */ { { -2.e+00f, -2.e+00f }, { 8.0471897e-01f, -2.0344439e+00f }, 2.5e-01f },
        /* 73 */ { { -3.6093321e-13f, -2.e+00f }, { 8.0471897e-01f, -1.1071488e+00f }, 5.e-01f },
        /* 74 */ { { -6.5136393e-26f, -2.e+00f }, { 8.0471897e-01f, -1.1071488e+00f }, 5.e-01f },
        /* 75 */ { { -min, -2.e+00f }, { 8.0471897e-01f, -1.1071488e+00f }, 5.e-01f },
        /* 76 */ { { zero, -2.e+00f }, { 8.0471897e-01f, -1.1071488e+00f }, 5.e-01f },
        /* 77 */ { { min, -2.e+00f }, { 8.0471897e-01f, -1.1071488e+00f }, 5.e-01f },
        /* 78 */ { { 6.5136393e-26f, -2.e+00f }, { 8.0471897e-01f, -1.1071488e+00f }, 5.e-01f },
        /* 79 */ { { 3.6093321e-13f, -2.e+00f }, { 8.0471897e-01f, -1.1071488e+00f }, 5.e-01f },
        /* 80 */ { { 2.e+00f, -2.e+00f }, { 1.2824746e+00f, -5.8800262e-01f }, 5.e-01f },
        /* 81 */ { { 1.1082383e+13f, -2.e+00f }, { 3.0036377e+01f, -1.804666e-13f }, 3.125e-02f },
        /* 82 */ { { 6.1409603e+25f, -2.e+00f }, { 5.9379608e+01f, -3.2568196e-26f }, 1.5625e-02f },
        /* 83 */ { { max, -2.e+00f }, { 8.8722839e+01f, -5.8774718e-39f }, 7.8125e-03f },
        /* 84 */ { { inf, -2.e+00f }, { inf, zero }, 1.e+00f },
        /* 85 */ { { -inf, -3.6093321e-13f }, { inf, -pi }, 1.e+00f },
        /* 86 */ { { -max, -3.6093321e-13f }, { 8.8722839e+01f, -pi }, 7.8125e-03f },
        /* 87 */ { { -6.1409603e+25f, -3.6093321e-13f }, { 5.9379608e+01f, -pi }, 1.5625e-02f },
        /* 88 */ { { -1.1082383e+13f, -3.6093321e-13f }, { 3.0036377e+01f, -pi }, 3.125e-02f },
        /* 89 */ { { -2.e+00f, -3.6093321e-13f }, { 6.5136393e-26f, -pi }, 2.5e-01f },
        /* 90 */ { { -3.6093321e-13f, -3.6093321e-13f }, { -3.6093321e-13f, -3.6093321e-13f }, 1.0995116e+12f },
        /* 91 */ { { -6.5136393e-26f, -3.6093321e-13f }, { -2.8437115e-33f, -3.6093321e-13f }, 2.1990233e+12f },
        /* 92 */ { { -min, -3.6093321e-13f }, { 6.5136393e-26f, -3.6093321e-13f }, 2.1990233e+12f },
        /* 93 */ { { zero, -3.6093321e-13f }, { 6.5136393e-26f, -3.6093321e-13f }, 2.1990233e+12f },
        /* 94 */ { { min, -3.6093321e-13f }, { 6.5136393e-26f, -3.6093321e-13f }, 2.1990233e+12f },
        /* 95 */ { { 6.5136393e-26f, -3.6093321e-13f }, { 1.3027279e-25f, -3.6093321e-13f }, 2.1990233e+12f },
        /* 96 */ { { 3.6093321e-13f, -3.6093321e-13f }, { 3.6093321e-13f, -3.6093321e-13f }, 1.0995116e+12f },
        /* 97 */ { { 2.e+00f, -3.6093321e-13f }, { 1.0986123e+00f, -1.2031107e-13f }, 5.e-01f },
        /* 98 */ { { 1.1082383e+13f, -3.6093321e-13f }, { 3.0036377e+01f, -3.2568193e-26f }, 3.125e-02f },
        /* 99 */ { { 6.1409603e+25f, -3.6093321e-13f }, { 5.9379608e+01f, -5.8774718e-39f }, 1.5625e-02f },
        /* 100 */ { { max, -3.6093321e-13f }, { 8.8722839e+01f, zero }, 7.8125e-03f },
        /* 101 */ { { inf, -3.6093321e-13f }, { inf, zero }, 1.e+00f },
        /* 102 */ { { -inf, -6.5136393e-26f }, { inf, -pi }, 1.e+00f },
        /* 103 */ { { -max, -6.5136393e-26f }, { 8.8722839e+01f, -pi }, 7.8125e-03f },
        /* 104 */ { { -6.1409603e+25f, -6.5136393e-26f }, { 5.9379608e+01f, -pi }, 1.5625e-02f },
        /* 105 */ { { -1.1082383e+13f, -6.5136393e-26f }, { 3.0036377e+01f, -pi }, 3.125e-02f },
        /* 106 */ { { -2.e+00f, -6.5136393e-26f }, { zero, -pi }, 2.5e-01f },
        /* 107 */ { { -3.6093321e-13f, -6.5136393e-26f }, { -3.6093321e-13f, -6.5136393e-26f }, 2.1990233e+12f },
        /* 108 */ { { -6.5136393e-26f, -6.5136393e-26f }, { -6.5136393e-26f, -6.5136393e-26f }, 9.6714066e+24f },
        /* 109 */ { { -min, -6.5136393e-26f }, { -min, -6.5136393e-26f }, 9.6714066e+24f },
        /* 110 */ { { zero, -6.5136393e-26f }, { zero, -6.5136393e-26f }, 9.6714066e+24f },
        /* 111 */ { { min, -6.5136393e-26f }, { min, -6.5136393e-26f }, 9.6714066e+24f },
        /* 112 */ { { 6.5136393e-26f, -6.5136393e-26f }, { 6.5136393e-26f, -6.5136393e-26f }, 9.6714066e+24f },
        /* 113 */ { { 3.6093321e-13f, -6.5136393e-26f }, { 3.6093321e-13f, -6.5136393e-26f }, 2.1990233e+12f },
        /* 114 */ { { 2.e+00f, -6.5136393e-26f }, { 1.0986123e+00f, -2.1712131e-26f }, 5.e-01f },
        /* 115 */ { { 1.1082383e+13f, -6.5136393e-26f }, { 3.0036377e+01f, -5.8774718e-39f }, 3.125e-02f },
        /* 116 */ { { 6.1409603e+25f, -6.5136393e-26f }, { 5.9379608e+01f, zero }, 1.5625e-02f },
        /* 117 */ { { max, -6.5136393e-26f }, { 8.8722839e+01f, zero }, 7.8125e-03f },
        /* 118 */ { { inf, -6.5136393e-26f }, { inf, zero }, 1.e+00f },
        /* 119 */ { { -inf, -min }, { inf, -pi }, 1.e+00f },
        /* 120 */ { { -max, -min }, { 8.8722839e+01f, -pi }, 7.8125e-03f },
        /* 121 */ { { -6.1409603e+25f, -min }, { 5.9379608e+01f, -pi }, 1.5625e-02f },
        /* 122 */ { { -1.1082383e+13f, -min }, { 3.0036377e+01f, -pi }, 3.125e-02f },
        /* 123 */ { { -2.e+00f, -min }, { zero, -pi }, 2.5e-01f },
        /* 124 */ { { -3.6093321e-13f, -min }, { -3.6093321e-13f, -min }, 2.1990233e+12f },
        /* 125 */ { { -6.5136393e-26f, -min }, { -6.5136393e-26f, -min }, 9.6714066e+24f },
        /* 126 */ { { -min, -min }, { -min, -min }, 4.2535296e+37f },
        /* 127 */ { { zero, -min }, { zero, -min }, 4.2535296e+37f },
        /* 128 */ { { min, -min }, { min, -min }, 4.2535296e+37f },
        /* 129 */ { { 6.5136393e-26f, -min }, { 6.5136393e-26f, -min }, 9.6714066e+24f },
        /* 130 */ { { 3.6093321e-13f, -min }, { 3.6093321e-13f, -min }, 2.1990233e+12f },
        /* 131 */ { { 2.e+00f, -min }, { 1.0986123e+00f, zero }, 5.e-01f },
        /* 132 */ { { 1.1082383e+13f, -min }, { 3.0036377e+01f, zero }, 3.125e-02f },
        /* 133 */ { { 6.1409603e+25f, -min }, { 5.9379608e+01f, zero }, 1.5625e-02f },
        /* 134 */ { { max, -min }, { 8.8722839e+01f, zero }, 7.8125e-03f },
        /* 135 */ { { inf, -min }, { inf, zero }, 1.e+00f },
        /* 136 */ { { -inf, zero }, { inf, pi }, 1.e+00f },
        /* 137 */ { { -max, zero }, { 8.8722839e+01f, pi }, 7.8125e-03f },
        /* 138 */ { { -6.1409603e+25f, zero }, { 5.9379608e+01f, pi }, 1.5625e-02f },
        /* 139 */ { { -1.1082383e+13f, zero }, { 3.0036377e+01f, pi }, 3.125e-02f },
        /* 140 */ { { -2.e+00f, zero }, { zero, pi }, 2.5e-01f },
        /* 141 */ { { -3.6093321e-13f, zero }, { -3.6093321e-13f, zero }, 2.1990233e+12f },
        /* 142 */ { { -6.5136393e-26f, zero }, { -6.5136393e-26f, zero }, 9.6714066e+24f },
        /* 143 */ { { -min, zero }, { -min, zero }, 4.2535296e+37f },
        /* 144 */ { { zero, zero }, { zero, zero }, 1.e+00f },
        /* 145 */ { { min, zero }, { min, zero }, 4.2535296e+37f },
        /* 146 */ { { 6.5136393e-26f, zero }, { 6.5136393e-26f, zero }, 9.6714066e+24f },
        /* 147 */ { { 3.6093321e-13f, zero }, { 3.6093321e-13f, zero }, 2.1990233e+12f },
        /* 148 */ { { 2.e+00f, zero }, { 1.0986123e+00f, zero }, 5.e-01f },
        /* 149 */ { { 1.1082383e+13f, zero }, { 3.0036377e+01f, zero }, 3.125e-02f },
        /* 150 */ { { 6.1409603e+25f, zero }, { 5.9379608e+01f, zero }, 1.5625e-02f },
        /* 151 */ { { max, zero }, { 8.8722839e+01f, zero }, 7.8125e-03f },
        /* 152 */ { { inf, zero }, { inf, zero }, 1.e+00f },
        /* 153 */ { { -inf, min }, { inf, pi }, 1.e+00f },
        /* 154 */ { { -max, min }, { 8.8722839e+01f, pi }, 7.8125e-03f },
        /* 155 */ { { -6.1409603e+25f, min }, { 5.9379608e+01f, pi }, 1.5625e-02f },
        /* 156 */ { { -1.1082383e+13f, min }, { 3.0036377e+01f, pi }, 3.125e-02f },
        /* 157 */ { { -2.e+00f, min }, { zero, pi }, 2.5e-01f },
        /* 158 */ { { -3.6093321e-13f, min }, { -3.6093321e-13f, min }, 2.1990233e+12f },
        /* 159 */ { { -6.5136393e-26f, min }, { -6.5136393e-26f, min }, 9.6714066e+24f },
        /* 160 */ { { -min, min }, { -min, min }, 4.2535296e+37f },
        /* 161 */ { { zero, min }, { zero, min }, 4.2535296e+37f },
        /* 162 */ { { min, min }, { min, min }, 4.2535296e+37f },
        /* 163 */ { { 6.5136393e-26f, min }, { 6.5136393e-26f, min }, 9.6714066e+24f },
        /* 164 */ { { 3.6093321e-13f, min }, { 3.6093321e-13f, min }, 2.1990233e+12f },
        /* 165 */ { { 2.e+00f, min }, { 1.0986123e+00f, zero }, 5.e-01f },
        /* 166 */ { { 1.1082383e+13f, min }, { 3.0036377e+01f, zero }, 3.125e-02f },
        /* 167 */ { { 6.1409603e+25f, min }, { 5.9379608e+01f, zero }, 1.5625e-02f },
        /* 168 */ { { max, min }, { 8.8722839e+01f, zero }, 7.8125e-03f },
        /* 169 */ { { inf, min }, { inf, zero }, 1.e+00f },
        /* 170 */ { { -inf, 6.5136393e-26f }, { inf, pi }, 1.e+00f },
        /* 171 */ { { -max, 6.5136393e-26f }, { 8.8722839e+01f, pi }, 7.8125e-03f },
        /* 172 */ { { -6.1409603e+25f, 6.5136393e-26f }, { 5.9379608e+01f, pi }, 1.5625e-02f },
        /* 173 */ { { -1.1082383e+13f, 6.5136393e-26f }, { 3.0036377e+01f, pi }, 3.125e-02f },
        /* 174 */ { { -2.e+00f, 6.5136393e-26f }, { zero, pi }, 2.5e-01f },
        /* 175 */ { { -3.6093321e-13f, 6.5136393e-26f }, { -3.6093321e-13f, 6.5136393e-26f }, 2.1990233e+12f },
        /* 176 */ { { -6.5136393e-26f, 6.5136393e-26f }, { -6.5136393e-26f, 6.5136393e-26f }, 9.6714066e+24f },
        /* 177 */ { { -min, 6.5136393e-26f }, { -min, 6.5136393e-26f }, 9.6714066e+24f },
        /* 178 */ { { zero, 6.5136393e-26f }, { zero, 6.5136393e-26f }, 9.6714066e+24f },
        /* 179 */ { { min, 6.5136393e-26f }, { min, 6.5136393e-26f }, 9.6714066e+24f },
        /* 180 */ { { 6.5136393e-26f, 6.5136393e-26f }, { 6.5136393e-26f, 6.5136393e-26f }, 9.6714066e+24f },
        /* 181 */ { { 3.6093321e-13f, 6.5136393e-26f }, { 3.6093321e-13f, 6.5136393e-26f }, 2.1990233e+12f },
        /* 182 */ { { 2.e+00f, 6.5136393e-26f }, { 1.0986123e+00f, 2.1712131e-26f }, 5.e-01f },
        /* 183 */ { { 1.1082383e+13f, 6.5136393e-26f }, { 3.0036377e+01f, 5.8774718e-39f }, 3.125e-02f },
        /* 184 */ { { 6.1409603e+25f, 6.5136393e-26f }, { 5.9379608e+01f, zero }, 1.5625e-02f },
        /* 185 */ { { max, 6.5136393e-26f }, { 8.8722839e+01f, zero }, 7.8125e-03f },
        /* 186 */ { { inf, 6.5136393e-26f }, { inf, zero }, 1.e+00f },
        /* 187 */ { { -inf, 3.6093321e-13f }, { inf, pi }, 1.e+00f },
        /* 188 */ { { -max, 3.6093321e-13f }, { 8.8722839e+01f, pi }, 7.8125e-03f },
        /* 189 */ { { -6.1409603e+25f, 3.6093321e-13f }, { 5.9379608e+01f, pi }, 1.5625e-02f },
        /* 190 */ { { -1.1082383e+13f, 3.6093321e-13f }, { 3.0036377e+01f, pi }, 3.125e-02f },
        /* 191 */ { { -2.e+00f, 3.6093321e-13f }, { 6.5136393e-26f, pi }, 2.5e-01f },
        /* 192 */ { { -3.6093321e-13f, 3.6093321e-13f }, { -3.6093321e-13f, 3.6093321e-13f }, 1.0995116e+12f },
        /* 193 */ { { -6.5136393e-26f, 3.6093321e-13f }, { -2.8437115e-33f, 3.6093321e-13f }, 2.1990233e+12f },
        /* 194 */ { { -min, 3.6093321e-13f }, { 6.5136393e-26f, 3.6093321e-13f }, 2.1990233e+12f },
        /* 195 */ { { zero, 3.6093321e-13f }, { 6.5136393e-26f, 3.6093321e-13f }, 2.1990233e+12f },
        /* 196 */ { { min, 3.6093321e-13f }, { 6.5136393e-26f, 3.6093321e-13f }, 2.1990233e+12f },
        /* 197 */ { { 6.5136393e-26f, 3.6093321e-13f }, { 1.3027279e-25f, 3.6093321e-13f }, 2.1990233e+12f },
        /* 198 */ { { 3.6093321e-13f, 3.6093321e-13f }, { 3.6093321e-13f, 3.6093321e-13f }, 1.0995116e+12f },
        /* 199 */ { { 2.e+00f, 3.6093321e-13f }, { 1.0986123e+00f, 1.2031107e-13f }, 5.e-01f },
        /* 200 */ { { 1.1082383e+13f, 3.6093321e-13f }, { 3.0036377e+01f, 3.2568193e-26f }, 3.125e-02f },
        /* 201 */ { { 6.1409603e+25f, 3.6093321e-13f }, { 5.9379608e+01f, 5.8774718e-39f }, 1.5625e-02f },
        /* 202 */ { { max, 3.6093321e-13f }, { 8.8722839e+01f, zero }, 7.8125e-03f },
        /* 203 */ { { inf, 3.6093321e-13f }, { inf, zero }, 1.e+00f },
        /* 204 */ { { -inf, 2.e+00f }, { inf, pi }, 1.e+00f },
        /* 205 */ { { -max, 2.e+00f }, { 8.8722839e+01f, pi }, 7.8125e-03f },
        /* 206 */ { { -6.1409603e+25f, 2.e+00f }, { 5.9379608e+01f, pi }, 1.5625e-02f },
        /* 207 */ { { -1.1082383e+13f, 2.e+00f }, { 3.0036377e+01f, pi }, 3.125e-02f },
        /* 208 */ { { -2.e+00f, 2.e+00f }, { 8.0471897e-01f, 2.0344439e+00f }, 2.5e-01f },
        /* 209 */ { { -3.6093321e-13f, 2.e+00f }, { 8.0471897e-01f, 1.1071488e+00f }, 5.e-01f },
        /* 210 */ { { -6.5136393e-26f, 2.e+00f }, { 8.0471897e-01f, 1.1071488e+00f }, 5.e-01f },
        /* 211 */ { { -min, 2.e+00f }, { 8.0471897e-01f, 1.1071488e+00f }, 5.e-01f },
        /* 212 */ { { zero, 2.e+00f }, { 8.0471897e-01f, 1.1071488e+00f }, 5.e-01f },
        /* 213 */ { { min, 2.e+00f }, { 8.0471897e-01f, 1.1071488e+00f }, 5.e-01f },
        /* 214 */ { { 6.5136393e-26f, 2.e+00f }, { 8.0471897e-01f, 1.1071488e+00f }, 5.e-01f },
        /* 215 */ { { 3.6093321e-13f, 2.e+00f }, { 8.0471897e-01f, 1.1071488e+00f }, 5.e-01f },
        /* 216 */ { { 2.e+00f, 2.e+00f }, { 1.2824746e+00f, 5.8800262e-01f }, 5.e-01f },
        /* 217 */ { { 1.1082383e+13f, 2.e+00f }, { 3.0036377e+01f, 1.804666e-13f }, 3.125e-02f },
        /* 218 */ { { 6.1409603e+25f, 2.e+00f }, { 5.9379608e+01f, 3.2568196e-26f }, 1.5625e-02f },
        /* 219 */ { { max, 2.e+00f }, { 8.8722839e+01f, 5.8774718e-39f }, 7.8125e-03f },
        /* 220 */ { { inf, 2.e+00f }, { inf, zero }, 1.e+00f },
        /* 221 */ { { -inf, 1.1082383e+13f }, { inf, pi }, 1.e+00f },
        /* 222 */ { { -max, 1.1082383e+13f }, { 8.8722839e+01f, pi }, 7.8125e-03f },
        /* 223 */ { { -6.1409603e+25f, 1.1082383e+13f }, { 5.9379608e+01f, pi }, 1.5625e-02f },
        /* 224 */ { { -1.1082383e+13f, 1.1082383e+13f }, { 3.0382952e+01f, pi3_4 }, 3.125e-02f },
        /* 225 */ { { -2.e+00f, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 226 */ { { -3.6093321e-13f, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 227 */ { { -6.5136393e-26f, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 228 */ { { -min, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 229 */ { { zero, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 230 */ { { min, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 231 */ { { 6.5136393e-26f, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 232 */ { { 3.6093321e-13f, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 233 */ { { 2.e+00f, 1.1082383e+13f }, { 3.0036377e+01f, pi_2 }, 3.125e-02f },
        /* 234 */ { { 1.1082383e+13f, 1.1082383e+13f }, { 3.0382952e+01f, pi_4 }, 3.125e-02f },
        /* 235 */ { { 6.1409603e+25f, 1.1082383e+13f }, { 5.9379608e+01f, 1.8046662e-13f }, 1.5625e-02f },
        /* 236 */ { { max, 1.1082383e+13f }, { 8.8722839e+01f, 3.25682e-26f }, 7.8125e-03f },
        /* 237 */ { { inf, 1.1082383e+13f }, { inf, zero }, 1.e+00f },
        /* 238 */ { { -inf, 6.1409603e+25f }, { inf, pi }, 1.e+00f },
        /* 239 */ { { -max, 6.1409603e+25f }, { 8.8722839e+01f, pi }, 7.8125e-03f },
        /* 240 */ { { -6.1409603e+25f, 6.1409603e+25f }, { 5.9726181e+01f, pi3_4 }, 1.5625e-02f },
        /* 241 */ { { -1.1082383e+13f, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 242 */ { { -2.e+00f, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 243 */ { { -3.6093321e-13f, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 244 */ { { -6.5136393e-26f, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 245 */ { { -min, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 246 */ { { zero, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 247 */ { { min, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 248 */ { { 6.5136393e-26f, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 249 */ { { 3.6093321e-13f, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 250 */ { { 2.e+00f, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 251 */ { { 1.1082383e+13f, 6.1409603e+25f }, { 5.9379608e+01f, pi_2 }, 1.5625e-02f },
        /* 252 */ { { 6.1409603e+25f, 6.1409603e+25f }, { 5.9726181e+01f, pi_4 }, 1.5625e-02f },
        /* 253 */ { { max, 6.1409603e+25f }, { 8.8722839e+01f, 1.8046662e-13f }, 7.8125e-03f },
        /* 254 */ { { inf, 6.1409603e+25f }, { inf, zero }, 1.e+00f },
        /* 255 */ { { -inf, max }, { inf, pi }, 1.e+00f },
        /* 256 */ { { -max, max }, { 8.9069412e+01f, pi3_4 }, 7.8125e-03f },
        /* 257 */ { { -6.1409603e+25f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 258 */ { { -1.1082383e+13f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 259 */ { { -2.e+00f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 260 */ { { -3.6093321e-13f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 261 */ { { -6.5136393e-26f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 262 */ { { -min, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 263 */ { { zero, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 264 */ { { min, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 265 */ { { 6.5136393e-26f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 266 */ { { 3.6093321e-13f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 267 */ { { 2.e+00f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 268 */ { { 1.1082383e+13f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 269 */ { { 6.1409603e+25f, max }, { 8.8722839e+01f, pi_2 }, 7.8125e-03f },
        /* 270 */ { { max, max }, { 8.9069412e+01f, pi_4 }, 7.8125e-03f },
        /* 271 */ { { inf, max }, { inf, zero }, 1.e+00f },
        /* 272 */ { { -inf, inf }, { inf, pi3_4 }, 1.e+00f },
        /* 273 */ { { -max, inf }, { inf, pi_2 }, 1.e+00f },
        /* 274 */ { { -6.1409603e+25f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 275 */ { { -1.1082383e+13f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 276 */ { { -2.e+00f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 277 */ { { -3.6093321e-13f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 278 */ { { -6.5136393e-26f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 279 */ { { -min, inf }, { inf, pi_2 }, 1.e+00f },
        /* 280 */ { { zero, inf }, { inf, pi_2 }, 1.e+00f },
        /* 281 */ { { min, inf }, { inf, pi_2 }, 1.e+00f },
        /* 282 */ { { 6.5136393e-26f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 283 */ { { 3.6093321e-13f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 284 */ { { 2.e+00f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 285 */ { { 1.1082383e+13f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 286 */ { { 6.1409603e+25f, inf }, { inf, pi_2 }, 1.e+00f },
        /* 287 */ { { max, inf }, { inf, pi_2 }, 1.e+00f },
        /* 288 */ { { inf, inf }, { inf, pi_4 }, 1.e+00f }
          // clang-format on
      };
      return table;
    } else if constexpr (std::is_same_v<T, double>) {
      const T pi = 3.141592653589793;
      const T pi_4 = 0.7853981633974483;
      const T pi_2 = 1.5707963267948966;
      const T pi3_4 = 2.356194490192345;
      const T zero = 0.0;
      const T inf = std::numeric_limits<T>::infinity();
      const T min = std::numeric_limits<T>::min();
      const T max = std::numeric_limits<T>::max();
      const TableType table{
          // clang-format off
          // Ignore max 80 character line width style requirement for
          // (i) the readability
          // (ii) the consistency with the local conventions
        /* 0 */ { { -inf, -inf }, { inf, -pi3_4 }, 1.e+00 },
        /* 1 */ { { -max, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 2 */ { { -4.0131652080900752e+205, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 3 */ { { -8.9589789687104559e+102, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 4 */ { { -1.9999999999998694e+00, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 5 */ { { -4.4647944971961829e-103, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 6 */ { { -9.9671949510973086e-206, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 7 */ { { -min, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 8 */ { { zero, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 9 */ { { min, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 10 */ { { 9.9671949510973086e-206, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 11 */ { { 4.4647944971961829e-103, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 12 */ { { 1.9999999999998694e+00, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 13 */ { { 8.9589789687104559e+102, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 14 */ { { 4.0131652080900752e+205, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 15 */ { { max, -inf }, { inf, -pi_2 }, 1.e+00 },
        /* 16 */ { { inf, -inf }, { inf, -pi_4 }, 1.e+00 },
        /* 17 */ { { -inf, -max }, { inf, -pi }, 1.e+00 },
        /* 18 */ { { -max, -max }, { 7.1012928648366392e+02, -pi3_4 }, 9.765625e-04 },
        /* 19 */ { { -4.0131652080900752e+205, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 20 */ { { -8.9589789687104559e+102, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 21 */ { { -1.9999999999998694e+00, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 22 */ { { -4.4647944971961829e-103, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 23 */ { { -9.9671949510973086e-206, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 24 */ { { -min, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 25 */ { { zero, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 26 */ { { min, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 27 */ { { 9.9671949510973086e-206, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 28 */ { { 4.4647944971961829e-103, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 29 */ { { 1.9999999999998694e+00, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 30 */ { { 8.9589789687104559e+102, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 31 */ { { 4.0131652080900752e+205, -max }, { 7.0978271289338397e+02, -pi_2 }, 9.765625e-04 },
        /* 32 */ { { max, -max }, { 7.1012928648366392e+02, -pi_4 }, 9.765625e-04 },
        /* 33 */ { { inf, -max }, { inf, zero }, 1.e+00 },
        /* 34 */ { { -inf, -4.0131652080900752e+205 }, { inf, -pi }, 1.e+00 },
        /* 35 */ { { -max, -4.0131652080900752e+205 }, { 7.0978271289338397e+02, -pi }, 9.765625e-04 },
        /* 36 */ { { -4.0131652080900752e+205, -4.0131652080900752e+205 }, { 4.737660979127225e+02, -pi3_4 }, 1.953125e-03 },
        /* 37 */ { { -8.9589789687104559e+102, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 38 */ { { -1.9999999999998694e+00, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 39 */ { { -4.4647944971961829e-103, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 40 */ { { -9.9671949510973086e-206, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 41 */ { { -min, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 42 */ { { zero, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 43 */ { { min, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 44 */ { { 9.9671949510973086e-206, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 45 */ { { 4.4647944971961829e-103, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 46 */ { { 1.9999999999998694e+00, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 47 */ { { 8.9589789687104559e+102, -4.0131652080900752e+205 }, { 4.7341952432244256e+02, -pi_2 }, 1.953125e-03 },
        /* 48 */ { { 4.0131652080900752e+205, -4.0131652080900752e+205 }, { 4.737660979127225e+02, -pi_4 }, 1.953125e-03 },
        /* 49 */ { { max, -4.0131652080900752e+205 }, { 7.0978271289338397e+02, -2.2323972485979601e-103 }, 9.765625e-04 },
        /* 50 */ { { inf, -4.0131652080900752e+205 }, { inf, zero }, 1.e+00 },
        /* 51 */ { { -inf, -8.9589789687104559e+102 }, { inf, -pi }, 1.e+00 },
        /* 52 */ { { -max, -8.9589789687104559e+102 }, { 7.0978271289338397e+02, -pi }, 9.765625e-04 },
        /* 53 */ { { -4.0131652080900752e+205, -8.9589789687104559e+102 }, { 4.7341952432244256e+02, -pi }, 1.953125e-03 },
        /* 54 */ { { -8.9589789687104559e+102, -8.9589789687104559e+102 }, { 2.374029093417812e+02, -pi3_4 }, 3.90625e-03 },
        /* 55 */ { { -1.9999999999998694e+00, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 56 */ { { -4.4647944971961829e-103, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 57 */ { { -9.9671949510973086e-206, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 58 */ { { -min, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 59 */ { { zero, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 60 */ { { min, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 61 */ { { 9.9671949510973086e-206, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 62 */ { { 4.4647944971961829e-103, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 63 */ { { 1.9999999999998694e+00, -8.9589789687104559e+102 }, { 2.3705633575150122e+02, -pi_2 }, 3.90625e-03 },
        /* 64 */ { { 8.9589789687104559e+102, -8.9589789687104559e+102 }, { 2.374029093417812e+02, -pi_4 }, 3.90625e-03 },
        /* 65 */ { { 4.0131652080900752e+205, -8.9589789687104559e+102 }, { 4.7341952432244256e+02, -2.2323972485982374e-103 }, 1.953125e-03 },
        /* 66 */ { { max, -8.9589789687104559e+102 }, { 7.0978271289338397e+02, -4.9835974755483611e-206 }, 9.765625e-04 },
        /* 67 */ { { inf, -8.9589789687104559e+102 }, { inf, zero }, 1.e+00 },
        /* 68 */ { { -inf, -1.9999999999998694e+00 }, { inf, -pi }, 1.e+00 },
        /* 69 */ { { -max, -1.9999999999998694e+00 }, { 7.0978271289338397e+02, -pi }, 9.765625e-04 },
        /* 70 */ { { -4.0131652080900752e+205, -1.9999999999998694e+00 }, { 4.7341952432244256e+02, -pi }, 1.953125e-03 },
        /* 71 */ { { -8.9589789687104559e+102, -1.9999999999998694e+00 }, { 2.3705633575150122e+02, -pi }, 3.90625e-03 },
        /* 72 */ { { -1.9999999999998694e+00, -1.9999999999998694e+00 }, { 8.0471895621697187e-01, -2.0344439357956765e+00 }, 2.5e-01 },
        /* 73 */ { { -4.4647944971961829e-103, -1.9999999999998694e+00 }, { 8.0471895621699796e-01, -1.1071487177940644e+00 }, 5.e-01 },
        /* 74 */ { { -9.9671949510973086e-206, -1.9999999999998694e+00 }, { 8.0471895621699796e-01, -1.1071487177940644e+00 }, 5.e-01 },
        /* 75 */ { { -min, -1.9999999999998694e+00 }, { 8.0471895621699796e-01, -1.1071487177940644e+00 }, 5.e-01 },
        /* 76 */ { { zero, -1.9999999999998694e+00 }, { 8.0471895621699796e-01, -1.1071487177940644e+00 }, 5.e-01 },
        /* 77 */ { { min, -1.9999999999998694e+00 }, { 8.0471895621699796e-01, -1.1071487177940644e+00 }, 5.e-01 },
        /* 78 */ { { 9.9671949510973086e-206, -1.9999999999998694e+00 }, { 8.0471895621699796e-01, -1.1071487177940644e+00 }, 5.e-01 },
        /* 79 */ { { 4.4647944971961829e-103, -1.9999999999998694e+00 }, { 8.0471895621699796e-01, -1.1071487177940644e+00 }, 5.e-01 },
        /* 80 */ { { 1.9999999999998694e+00, -1.9999999999998694e+00 }, { 1.2824746787307182e+00, -5.8800260354755751e-01 }, 5.e-01 },
        /* 81 */ { { 8.9589789687104559e+102, -1.9999999999998694e+00 }, { 2.3705633575150122e+02, -2.2323972485982374e-103 }, 3.90625e-03 },
        /* 82 */ { { 4.0131652080900752e+205, -1.9999999999998694e+00 }, { 4.7341952432244256e+02, -4.9835974755489796e-206 }, 1.953125e-03 },
        /* 83 */ { { max, -1.9999999999998694e+00 }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 84 */ { { inf, -1.9999999999998694e+00 }, { inf, zero }, 1.e+00 },
        /* 85 */ { { -inf, -4.4647944971961829e-103 }, { inf, -pi }, 1.e+00 },
        /* 86 */ { { -max, -4.4647944971961829e-103 }, { 7.0978271289338397e+02, -pi }, 9.765625e-04 },
        /* 87 */ { { -4.0131652080900752e+205, -4.4647944971961829e-103 }, { 4.7341952432244256e+02, -pi }, 1.953125e-03 },
        /* 88 */ { { -8.9589789687104559e+102, -4.4647944971961829e-103 }, { 2.3705633575150122e+02, -pi }, 3.90625e-03 },
        /* 89 */ { { -1.9999999999998694e+00, -4.4647944971961829e-103 }, { -1.3056222769592694e-13, -pi }, 2.5e-01 },
        /* 90 */ { { -4.4647944971961829e-103, -4.4647944971961829e-103 }, { -4.4647944971961829e-103, -4.4647944971961829e-103 }, 1.1198723710889021e+102 },
        /* 91 */ { { -9.9671949510973086e-206, -4.4647944971961829e-103 }, { -6.5066958834738373e-219, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 92 */ { { -min, -4.4647944971961829e-103 }, { 9.9671949510966579e-206, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 93 */ { { zero, -4.4647944971961829e-103 }, { 9.9671949510966579e-206, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 94 */ { { min, -4.4647944971961829e-103 }, { 9.9671949510966579e-206, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 95 */ { { 9.9671949510973086e-206, -4.4647944971961829e-103 }, { 1.9934389902193967e-205, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 96 */ { { 4.4647944971961829e-103, -4.4647944971961829e-103 }, { 4.4647944971961829e-103, -4.4647944971961829e-103 }, 1.1198723710889021e+102 },
        /* 97 */ { { 1.9999999999998694e+00, -4.4647944971961829e-103 }, { 1.0986122886680663e+00, -1.4882648323987925e-103 }, 5.e-01 },
        /* 98 */ { { 8.9589789687104559e+102, -4.4647944971961829e-103 }, { 2.3705633575150122e+02, -4.9835974755489796e-206 }, 3.90625e-03 },
        /* 99 */ { { 4.0131652080900752e+205, -4.4647944971961829e-103 }, { 4.7341952432244256e+02, -1.1125369292536664e-308 }, 1.953125e-03 },
        /* 100 */ { { max, -4.4647944971961829e-103 }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 101 */ { { inf, -4.4647944971961829e-103 }, { inf, zero }, 1.e+00 },
        /* 102 */ { { -inf, -9.9671949510973086e-206 }, { inf, -pi }, 1.e+00 },
        /* 103 */ { { -max, -9.9671949510973086e-206 }, { 7.0978271289338397e+02, -pi }, 9.765625e-04 },
        /* 104 */ { { -4.0131652080900752e+205, -9.9671949510973086e-206 }, { 4.7341952432244256e+02, -pi }, 1.953125e-03 },
        /* 105 */ { { -8.9589789687104559e+102, -9.9671949510973086e-206 }, { 2.3705633575150122e+02, -pi }, 3.90625e-03 },
        /* 106 */ { { -1.9999999999998694e+00, -9.9671949510973086e-206 }, { -1.3056222769592694e-13, -pi }, 2.5e-01 },
        /* 107 */ { { -4.4647944971961829e-103, -9.9671949510973086e-206 }, { -4.4647944971961829e-103, -9.9671949510973086e-206 }, 2.2397447421778042e+102 },
        /* 108 */ { { -9.9671949510973086e-206, -9.9671949510973086e-206 }, { -9.9671949510973086e-206, -9.9671949510973086e-206 }, 5.0164565101131187e+204 },
        /* 109 */ { { -min, -9.9671949510973086e-206 }, { -min, -9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 110 */ { { zero, -9.9671949510973086e-206 }, { zero, -9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 111 */ { { min, -9.9671949510973086e-206 }, { min, -9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 112 */ { { 9.9671949510973086e-206, -9.9671949510973086e-206 }, { 9.9671949510973086e-206, -9.9671949510973086e-206 }, 5.0164565101131187e+204 },
        /* 113 */ { { 4.4647944971961829e-103, -9.9671949510973086e-206 }, { 4.4647944971961829e-103, -9.9671949510973086e-206 }, 2.2397447421778042e+102 },
        /* 114 */ { { 1.9999999999998694e+00, -9.9671949510973086e-206 }, { 1.0986122886680663e+00, -3.322398317032581e-206 }, 5.e-01 },
        /* 115 */ { { 8.9589789687104559e+102, -9.9671949510973086e-206 }, { 2.3705633575150122e+02, -1.1125369292536664e-308 }, 3.90625e-03 },
        /* 116 */ { { 4.0131652080900752e+205, -9.9671949510973086e-206 }, { 4.7341952432244256e+02, zero }, 1.953125e-03 },
        /* 117 */ { { max, -9.9671949510973086e-206 }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 118 */ { { inf, -9.9671949510973086e-206 }, { inf, zero }, 1.e+00 },
        /* 119 */ { { -inf, -min }, { inf, -pi }, 1.e+00 },
        /* 120 */ { { -max, -min }, { 7.0978271289338397e+02, -pi }, 9.765625e-04 },
        /* 121 */ { { -4.0131652080900752e+205, -min }, { 4.7341952432244256e+02, -pi }, 1.953125e-03 },
        /* 122 */ { { -8.9589789687104559e+102, -min }, { 2.3705633575150122e+02, -pi }, 3.90625e-03 },
        /* 123 */ { { -1.9999999999998694e+00, -min }, { -1.3056222769592694e-13, -pi }, 2.5e-01 },
        /* 124 */ { { -4.4647944971961829e-103, -min }, { -4.4647944971961829e-103, -min }, 2.2397447421778042e+102 },
        /* 125 */ { { -9.9671949510973086e-206, -min }, { -9.9671949510973086e-206, -min }, 1.0032913020226237e+205 },
        /* 126 */ { { -min, -min }, { -min, -min }, 2.2471164185778949e+307 },
        /* 127 */ { { zero, -min }, { zero, -min }, 2.2471164185778949e+307 },
        /* 128 */ { { min, -min }, { min, -min }, 2.2471164185778949e+307 },
        /* 129 */ { { 9.9671949510973086e-206, -min }, { 9.9671949510973086e-206, -min }, 1.0032913020226237e+205 },
        /* 130 */ { { 4.4647944971961829e-103, -min }, { 4.4647944971961829e-103, -min }, 2.2397447421778042e+102 },
        /* 131 */ { { 1.9999999999998694e+00, -min }, { 1.0986122886680663e+00, zero }, 5.e-01 },
        /* 132 */ { { 8.9589789687104559e+102, -min }, { 2.3705633575150122e+02, zero }, 3.90625e-03 },
        /* 133 */ { { 4.0131652080900752e+205, -min }, { 4.7341952432244256e+02, zero }, 1.953125e-03 },
        /* 134 */ { { max, -min }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 135 */ { { inf, -min }, { inf, zero }, 1.e+00 },
        /* 136 */ { { -inf, zero }, { inf, pi }, 1.e+00 },
        /* 137 */ { { -max, zero }, { 7.0978271289338397e+02, pi }, 9.765625e-04 },
        /* 138 */ { { -4.0131652080900752e+205, zero }, { 4.7341952432244256e+02, pi }, 1.953125e-03 },
        /* 139 */ { { -8.9589789687104559e+102, zero }, { 2.3705633575150122e+02, pi }, 3.90625e-03 },
        /* 140 */ { { -1.9999999999998694e+00, zero }, { -1.3056222769592694e-13, pi }, 2.5e-01 },
        /* 141 */ { { -4.4647944971961829e-103, zero }, { -4.4647944971961829e-103, zero }, 2.2397447421778042e+102 },
        /* 142 */ { { -9.9671949510973086e-206, zero }, { -9.9671949510973086e-206, zero }, 1.0032913020226237e+205 },
        /* 143 */ { { -min, zero }, { -min, zero }, 2.2471164185778949e+307 },
        /* 144 */ { { zero, zero }, { zero, zero }, 1.e+00 },
        /* 145 */ { { min, zero }, { min, zero }, 2.2471164185778949e+307 },
        /* 146 */ { { 9.9671949510973086e-206, zero }, { 9.9671949510973086e-206, zero }, 1.0032913020226237e+205 },
        /* 147 */ { { 4.4647944971961829e-103, zero }, { 4.4647944971961829e-103, zero }, 2.2397447421778042e+102 },
        /* 148 */ { { 1.9999999999998694e+00, zero }, { 1.0986122886680663e+00, zero }, 5.e-01 },
        /* 149 */ { { 8.9589789687104559e+102, zero }, { 2.3705633575150122e+02, zero }, 3.90625e-03 },
        /* 150 */ { { 4.0131652080900752e+205, zero }, { 4.7341952432244256e+02, zero }, 1.953125e-03 },
        /* 151 */ { { max, zero }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 152 */ { { inf, zero }, { inf, zero }, 1.e+00 },
        /* 153 */ { { -inf, min }, { inf, pi }, 1.e+00 },
        /* 154 */ { { -max, min }, { 7.0978271289338397e+02, pi }, 9.765625e-04 },
        /* 155 */ { { -4.0131652080900752e+205, min }, { 4.7341952432244256e+02, pi }, 1.953125e-03 },
        /* 156 */ { { -8.9589789687104559e+102, min }, { 2.3705633575150122e+02, pi }, 3.90625e-03 },
        /* 157 */ { { -1.9999999999998694e+00, min }, { -1.3056222769592694e-13, pi }, 2.5e-01 },
        /* 158 */ { { -4.4647944971961829e-103, min }, { -4.4647944971961829e-103, min }, 2.2397447421778042e+102 },
        /* 159 */ { { -9.9671949510973086e-206, min }, { -9.9671949510973086e-206, min }, 1.0032913020226237e+205 },
        /* 160 */ { { -min, min }, { -min, min }, 2.2471164185778949e+307 },
        /* 161 */ { { zero, min }, { zero, min }, 2.2471164185778949e+307 },
        /* 162 */ { { min, min }, { min, min }, 2.2471164185778949e+307 },
        /* 163 */ { { 9.9671949510973086e-206, min }, { 9.9671949510973086e-206, min }, 1.0032913020226237e+205 },
        /* 164 */ { { 4.4647944971961829e-103, min }, { 4.4647944971961829e-103, min }, 2.2397447421778042e+102 },
        /* 165 */ { { 1.9999999999998694e+00, min }, { 1.0986122886680663e+00, zero }, 5.e-01 },
        /* 166 */ { { 8.9589789687104559e+102, min }, { 2.3705633575150122e+02, zero }, 3.90625e-03 },
        /* 167 */ { { 4.0131652080900752e+205, min }, { 4.7341952432244256e+02, zero }, 1.953125e-03 },
        /* 168 */ { { max, min }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 169 */ { { inf, min }, { inf, zero }, 1.e+00 },
        /* 170 */ { { -inf, 9.9671949510973086e-206 }, { inf, pi }, 1.e+00 },
        /* 171 */ { { -max, 9.9671949510973086e-206 }, { 7.0978271289338397e+02, pi }, 9.765625e-04 },
        /* 172 */ { { -4.0131652080900752e+205, 9.9671949510973086e-206 }, { 4.7341952432244256e+02, pi }, 1.953125e-03 },
        /* 173 */ { { -8.9589789687104559e+102, 9.9671949510973086e-206 }, { 2.3705633575150122e+02, pi }, 3.90625e-03 },
        /* 174 */ { { -1.9999999999998694e+00, 9.9671949510973086e-206 }, { -1.3056222769592694e-13, pi }, 2.5e-01 },
        /* 175 */ { { -4.4647944971961829e-103, 9.9671949510973086e-206 }, { -4.4647944971961829e-103, 9.9671949510973086e-206 }, 2.2397447421778042e+102 },
        /* 176 */ { { -9.9671949510973086e-206, 9.9671949510973086e-206 }, { -9.9671949510973086e-206, 9.9671949510973086e-206 }, 5.0164565101131187e+204 },
        /* 177 */ { { -min, 9.9671949510973086e-206 }, { -min, 9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 178 */ { { zero, 9.9671949510973086e-206 }, { zero, 9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 179 */ { { min, 9.9671949510973086e-206 }, { min, 9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 180 */ { { 9.9671949510973086e-206, 9.9671949510973086e-206 }, { 9.9671949510973086e-206, 9.9671949510973086e-206 }, 5.0164565101131187e+204 },
        /* 181 */ { { 4.4647944971961829e-103, 9.9671949510973086e-206 }, { 4.4647944971961829e-103, 9.9671949510973086e-206 }, 2.2397447421778042e+102 },
        /* 182 */ { { 1.9999999999998694e+00, 9.9671949510973086e-206 }, { 1.0986122886680663e+00, 3.322398317032581e-206 }, 5.e-01 },
        /* 183 */ { { 8.9589789687104559e+102, 9.9671949510973086e-206 }, { 2.3705633575150122e+02, 1.1125369292536664e-308 }, 3.90625e-03 },
        /* 184 */ { { 4.0131652080900752e+205, 9.9671949510973086e-206 }, { 4.7341952432244256e+02, zero }, 1.953125e-03 },
        /* 185 */ { { max, 9.9671949510973086e-206 }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 186 */ { { inf, 9.9671949510973086e-206 }, { inf, zero }, 1.e+00 },
        /* 187 */ { { -inf, 4.4647944971961829e-103 }, { inf, pi }, 1.e+00 },
        /* 188 */ { { -max, 4.4647944971961829e-103 }, { 7.0978271289338397e+02, pi }, 9.765625e-04 },
        /* 189 */ { { -4.0131652080900752e+205, 4.4647944971961829e-103 }, { 4.7341952432244256e+02, pi }, 1.953125e-03 },
        /* 190 */ { { -8.9589789687104559e+102, 4.4647944971961829e-103 }, { 2.3705633575150122e+02, pi }, 3.90625e-03 },
        /* 191 */ { { -1.9999999999998694e+00, 4.4647944971961829e-103 }, { -1.3056222769592694e-13, pi }, 2.5e-01 },
        /* 192 */ { { -4.4647944971961829e-103, 4.4647944971961829e-103 }, { -4.4647944971961829e-103, 4.4647944971961829e-103 }, 1.1198723710889021e+102 },
        /* 193 */ { { -9.9671949510973086e-206, 4.4647944971961829e-103 }, { -6.5066958834738373e-219, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 194 */ { { -min, 4.4647944971961829e-103 }, { 9.9671949510966579e-206, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 195 */ { { zero, 4.4647944971961829e-103 }, { 9.9671949510966579e-206, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 196 */ { { min, 4.4647944971961829e-103 }, { 9.9671949510966579e-206, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 197 */ { { 9.9671949510973086e-206, 4.4647944971961829e-103 }, { 1.9934389902193967e-205, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 198 */ { { 4.4647944971961829e-103, 4.4647944971961829e-103 }, { 4.4647944971961829e-103, 4.4647944971961829e-103 }, 1.1198723710889021e+102 },
        /* 199 */ { { 1.9999999999998694e+00, 4.4647944971961829e-103 }, { 1.0986122886680663e+00, 1.4882648323987925e-103 }, 5.e-01 },
        /* 200 */ { { 8.9589789687104559e+102, 4.4647944971961829e-103 }, { 2.3705633575150122e+02, 4.9835974755489796e-206 }, 3.90625e-03 },
        /* 201 */ { { 4.0131652080900752e+205, 4.4647944971961829e-103 }, { 4.7341952432244256e+02, 1.1125369292536664e-308 }, 1.953125e-03 },
        /* 202 */ { { max, 4.4647944971961829e-103 }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 203 */ { { inf, 4.4647944971961829e-103 }, { inf, zero }, 1.e+00 },
        /* 204 */ { { -inf, 1.9999999999998694e+00 }, { inf, pi }, 1.e+00 },
        /* 205 */ { { -max, 1.9999999999998694e+00 }, { 7.0978271289338397e+02, pi }, 9.765625e-04 },
        /* 206 */ { { -4.0131652080900752e+205, 1.9999999999998694e+00 }, { 4.7341952432244256e+02, pi }, 1.953125e-03 },
        /* 207 */ { { -8.9589789687104559e+102, 1.9999999999998694e+00 }, { 2.3705633575150122e+02, pi }, 3.90625e-03 },
        /* 208 */ { { -1.9999999999998694e+00, 1.9999999999998694e+00 }, { 8.0471895621697187e-01, 2.0344439357956765e+00 }, 2.5e-01 },
        /* 209 */ { { -4.4647944971961829e-103, 1.9999999999998694e+00 }, { 8.0471895621699796e-01, 1.1071487177940644e+00 }, 5.e-01 },
        /* 210 */ { { -9.9671949510973086e-206, 1.9999999999998694e+00 }, { 8.0471895621699796e-01, 1.1071487177940644e+00 }, 5.e-01 },
        /* 211 */ { { -min, 1.9999999999998694e+00 }, { 8.0471895621699796e-01, 1.1071487177940644e+00 }, 5.e-01 },
        /* 212 */ { { zero, 1.9999999999998694e+00 }, { 8.0471895621699796e-01, 1.1071487177940644e+00 }, 5.e-01 },
        /* 213 */ { { min, 1.9999999999998694e+00 }, { 8.0471895621699796e-01, 1.1071487177940644e+00 }, 5.e-01 },
        /* 214 */ { { 9.9671949510973086e-206, 1.9999999999998694e+00 }, { 8.0471895621699796e-01, 1.1071487177940644e+00 }, 5.e-01 },
        /* 215 */ { { 4.4647944971961829e-103, 1.9999999999998694e+00 }, { 8.0471895621699796e-01, 1.1071487177940644e+00 }, 5.e-01 },
        /* 216 */ { { 1.9999999999998694e+00, 1.9999999999998694e+00 }, { 1.2824746787307182e+00, 5.8800260354755751e-01 }, 5.e-01 },
        /* 217 */ { { 8.9589789687104559e+102, 1.9999999999998694e+00 }, { 2.3705633575150122e+02, 2.2323972485982374e-103 }, 3.90625e-03 },
        /* 218 */ { { 4.0131652080900752e+205, 1.9999999999998694e+00 }, { 4.7341952432244256e+02, 4.9835974755489796e-206 }, 1.953125e-03 },
        /* 219 */ { { max, 1.9999999999998694e+00 }, { 7.0978271289338397e+02, zero }, 9.765625e-04 },
        /* 220 */ { { inf, 1.9999999999998694e+00 }, { inf, zero }, 1.e+00 },
        /* 221 */ { { -inf, 8.9589789687104559e+102 }, { inf, pi }, 1.e+00 },
        /* 222 */ { { -max, 8.9589789687104559e+102 }, { 7.0978271289338397e+02, pi }, 9.765625e-04 },
        /* 223 */ { { -4.0131652080900752e+205, 8.9589789687104559e+102 }, { 4.7341952432244256e+02, pi }, 1.953125e-03 },
        /* 224 */ { { -8.9589789687104559e+102, 8.9589789687104559e+102 }, { 2.374029093417812e+02, pi3_4 }, 3.90625e-03 },
        /* 225 */ { { -1.9999999999998694e+00, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 226 */ { { -4.4647944971961829e-103, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 227 */ { { -9.9671949510973086e-206, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 228 */ { { -min, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 229 */ { { zero, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 230 */ { { min, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 231 */ { { 9.9671949510973086e-206, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 232 */ { { 4.4647944971961829e-103, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 233 */ { { 1.9999999999998694e+00, 8.9589789687104559e+102 }, { 2.3705633575150122e+02, pi_2 }, 3.90625e-03 },
        /* 234 */ { { 8.9589789687104559e+102, 8.9589789687104559e+102 }, { 2.374029093417812e+02, pi_4 }, 3.90625e-03 },
        /* 235 */ { { 4.0131652080900752e+205, 8.9589789687104559e+102 }, { 4.7341952432244256e+02, 2.2323972485982374e-103 }, 1.953125e-03 },
        /* 236 */ { { max, 8.9589789687104559e+102 }, { 7.0978271289338397e+02, 4.9835974755483611e-206 }, 9.765625e-04 },
        /* 237 */ { { inf, 8.9589789687104559e+102 }, { inf, zero }, 1.e+00 },
        /* 238 */ { { -inf, 4.0131652080900752e+205 }, { inf, pi }, 1.e+00 },
        /* 239 */ { { -max, 4.0131652080900752e+205 }, { 7.0978271289338397e+02, pi }, 9.765625e-04 },
        /* 240 */ { { -4.0131652080900752e+205, 4.0131652080900752e+205 }, { 4.737660979127225e+02, pi3_4 }, 1.953125e-03 },
        /* 241 */ { { -8.9589789687104559e+102, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 242 */ { { -1.9999999999998694e+00, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 243 */ { { -4.4647944971961829e-103, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 244 */ { { -9.9671949510973086e-206, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 245 */ { { -min, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 246 */ { { zero, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 247 */ { { min, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 248 */ { { 9.9671949510973086e-206, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 249 */ { { 4.4647944971961829e-103, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 250 */ { { 1.9999999999998694e+00, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 251 */ { { 8.9589789687104559e+102, 4.0131652080900752e+205 }, { 4.7341952432244256e+02, pi_2 }, 1.953125e-03 },
        /* 252 */ { { 4.0131652080900752e+205, 4.0131652080900752e+205 }, { 4.737660979127225e+02, pi_4 }, 1.953125e-03 },
        /* 253 */ { { max, 4.0131652080900752e+205 }, { 7.0978271289338397e+02, 2.2323972485979601e-103 }, 9.765625e-04 },
        /* 254 */ { { inf, 4.0131652080900752e+205 }, { inf, zero }, 1.e+00 },
        /* 255 */ { { -inf, max }, { inf, pi }, 1.e+00 },
        /* 256 */ { { -max, max }, { 7.1012928648366392e+02, pi3_4 }, 9.765625e-04 },
        /* 257 */ { { -4.0131652080900752e+205, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 258 */ { { -8.9589789687104559e+102, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 259 */ { { -1.9999999999998694e+00, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 260 */ { { -4.4647944971961829e-103, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 261 */ { { -9.9671949510973086e-206, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 262 */ { { -min, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 263 */ { { zero, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 264 */ { { min, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 265 */ { { 9.9671949510973086e-206, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 266 */ { { 4.4647944971961829e-103, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 267 */ { { 1.9999999999998694e+00, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 268 */ { { 8.9589789687104559e+102, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 269 */ { { 4.0131652080900752e+205, max }, { 7.0978271289338397e+02, pi_2 }, 9.765625e-04 },
        /* 270 */ { { max, max }, { 7.1012928648366392e+02, pi_4 }, 9.765625e-04 },
        /* 271 */ { { inf, max }, { inf, zero }, 1.e+00 },
        /* 272 */ { { -inf, inf }, { inf, pi3_4 }, 1.e+00 },
        /* 273 */ { { -max, inf }, { inf, pi_2 }, 1.e+00 },
        /* 274 */ { { -4.0131652080900752e+205, inf }, { inf, pi_2 }, 1.e+00 },
        /* 275 */ { { -8.9589789687104559e+102, inf }, { inf, pi_2 }, 1.e+00 },
        /* 276 */ { { -1.9999999999998694e+00, inf }, { inf, pi_2 }, 1.e+00 },
        /* 277 */ { { -4.4647944971961829e-103, inf }, { inf, pi_2 }, 1.e+00 },
        /* 278 */ { { -9.9671949510973086e-206, inf }, { inf, pi_2 }, 1.e+00 },
        /* 279 */ { { -min, inf }, { inf, pi_2 }, 1.e+00 },
        /* 280 */ { { zero, inf }, { inf, pi_2 }, 1.e+00 },
        /* 281 */ { { min, inf }, { inf, pi_2 }, 1.e+00 },
        /* 282 */ { { 9.9671949510973086e-206, inf }, { inf, pi_2 }, 1.e+00 },
        /* 283 */ { { 4.4647944971961829e-103, inf }, { inf, pi_2 }, 1.e+00 },
        /* 284 */ { { 1.9999999999998694e+00, inf }, { inf, pi_2 }, 1.e+00 },
        /* 285 */ { { 8.9589789687104559e+102, inf }, { inf, pi_2 }, 1.e+00 },
        /* 286 */ { { 4.0131652080900752e+205, inf }, { inf, pi_2 }, 1.e+00 },
        /* 287 */ { { max, inf }, { inf, pi_2 }, 1.e+00 },
        /* 288 */ { { inf, inf }, { inf, pi_4 }, 1.e+00 }
          // clang-format on
      };
      return table;
    } else {
      static_assert(dependent_false<T>); /* unreachable */
    }
  }
};

template <typename T, int default_dps_deficiency = 0>
struct Tan {
  typedef std::complex<T> InputType;
  typedef std::complex<T> OutputType;
  typedef T FloatType;
  using TableType = std::vector<std::tuple<InputType, OutputType, FloatType>>;
  static constexpr int dps_deficiency = default_dps_deficiency;
  const TableType get() {
    if constexpr (std::is_same_v<T, float>) {
      const T zero = 0.0f;
      const T min = std::numeric_limits<T>::min();
      const T max = std::numeric_limits<T>::max();
      const TableType table{
          // clang-format off
          // Ignore max 80 character line width style requirement for
          // (i) the readability
          // (ii) the consistency with the local conventions
        // { { -inf, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { -max, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { -6.1409603e+25f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { -1.1082383e+13f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { -2.e+00f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { -3.6093321e-13f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { -6.5136393e-26f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { -min, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { zero, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { min, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { 6.5136393e-26f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { 3.6093321e-13f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { 2.e+00f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { 1.1082383e+13f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { 6.1409603e+25f, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { max, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { inf, -inf }, { zero, -1.e+00f }, 5.e-01f },
        // { { -inf, -max }, { nan, nan }, 1.e+00f },
        /* 0 */ { { -max, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 1 */ { { -6.1409603e+25f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 2 */ { { -1.1082383e+13f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 3 */ { { -2.e+00f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 4 */ { { -3.6093321e-13f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 5 */ { { -6.5136393e-26f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 6 */ { { -min, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 7 */ { { zero, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 8 */ { { min, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 9 */ { { 6.5136393e-26f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 10 */ { { 3.6093321e-13f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 11 */ { { 2.e+00f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 12 */ { { 1.1082383e+13f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 13 */ { { 6.1409603e+25f, -max }, { zero, -1.e+00f }, 5.e-01f },
        /* 14 */ { { max, -max }, { zero, -1.e+00f }, 5.e-01f },
        // { { inf, -max }, { nan, nan }, 1.e+00f },
        // { { -inf, -6.1409603e+25f }, { nan, nan }, 1.e+00f },
        /* 15 */ { { -max, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 16 */ { { -6.1409603e+25f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 17 */ { { -1.1082383e+13f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 18 */ { { -2.e+00f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 19 */ { { -3.6093321e-13f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 20 */ { { -6.5136393e-26f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 21 */ { { -min, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 22 */ { { zero, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 23 */ { { min, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 24 */ { { 6.5136393e-26f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 25 */ { { 3.6093321e-13f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 26 */ { { 2.e+00f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 27 */ { { 1.1082383e+13f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 28 */ { { 6.1409603e+25f, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        /* 29 */ { { max, -6.1409603e+25f }, { zero, -1.e+00f }, 5.e-01f },
        // { { inf, -6.1409603e+25f }, { nan, nan }, 1.e+00f },
        // { { -inf, -1.1082383e+13f }, { nan, nan }, 1.e+00f },
        /* 30 */ { { -max, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 31 */ { { -6.1409603e+25f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 32 */ { { -1.1082383e+13f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 33 */ { { -2.e+00f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 34 */ { { -3.6093321e-13f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 35 */ { { -6.5136393e-26f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 36 */ { { -min, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 37 */ { { zero, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 38 */ { { min, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 39 */ { { 6.5136393e-26f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 40 */ { { 3.6093321e-13f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 41 */ { { 2.e+00f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 42 */ { { 1.1082383e+13f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 43 */ { { 6.1409603e+25f, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        /* 44 */ { { max, -1.1082383e+13f }, { zero, -1.e+00f }, 5.e-01f },
        // { { inf, -1.1082383e+13f }, { nan, nan }, 1.e+00f },
        // { { -inf, -2.e+00f }, { nan, nan }, 1.e+00f },
        /* 45 */ { { -max, -2.e+00f }, { 3.2068815e-02f, -9.8294145e-01f }, 1.e+00f },
        /* 46 */ { { -6.1409603e+25f, -2.e+00f }, { 5.3751757e-03f, -1.0369183e+00f }, 5.e-01f },
        /* 47 */ { { -1.1082383e+13f, -2.e+00f }, { -3.4068439e-02f, -9.8717695e-01f }, 1.e+00f },
        /* 48 */ { { -2.e+00f, -2.e+00f }, { 2.8392954e-02f, -1.0238355e+00f }, 5.e-01f },
        /* 49 */ { { -3.6093321e-13f, -2.e+00f }, { -2.5500228e-14f, -9.6402758e-01f }, 1.e+00f },
        /* 50 */ { { -6.5136393e-26f, -2.e+00f }, { -4.6019399e-27f, -9.6402758e-01f }, 1.e+00f },
        /* 51 */ { { -min, -2.e+00f }, { zero, -9.6402758e-01f }, 1.e+00f },
        /* 52 */ { { zero, -2.e+00f }, { zero, -9.6402758e-01f }, 1.e+00f },
        /* 53 */ { { min, -2.e+00f }, { zero, -9.6402758e-01f }, 1.e+00f },
        /* 54 */ { { 6.5136393e-26f, -2.e+00f }, { 4.6019399e-27f, -9.6402758e-01f }, 1.e+00f },
        /* 55 */ { { 3.6093321e-13f, -2.e+00f }, { 2.5500228e-14f, -9.6402758e-01f }, 1.e+00f },
        /* 56 */ { { 2.e+00f, -2.e+00f }, { -2.8392954e-02f, -1.0238355e+00f }, 5.e-01f },
        /* 57 */ { { 1.1082383e+13f, -2.e+00f }, { 3.4068439e-02f, -9.8717695e-01f }, 1.e+00f },
        /* 58 */ { { 6.1409603e+25f, -2.e+00f }, { -5.3751757e-03f, -1.0369183e+00f }, 5.e-01f },
        /* 59 */ { { max, -2.e+00f }, { -3.2068815e-02f, -9.8294145e-01f }, 1.e+00f },
        // { { inf, -2.e+00f }, { nan, nan }, 1.e+00f },
        // { { -inf, -3.6093321e-13f }, { nan, nan }, 1.e+00f },
        /* 60 */ { { -max, -3.6093321e-13f }, { 6.1179793e-01f, -4.9602932e-13f }, 1.e+00f },
        /* 61 */ { { -6.1409603e+25f, -3.6093321e-13f }, { 1.406664e+01f, -7.1778909e-11f }, 6.25e-02f },
        /* 62 */ { { -1.1082383e+13f, -3.6093321e-13f }, { -7.0485216e-01f, -5.4025084e-13f }, 1.e+00f },
        /* 63 */ { { -2.e+00f, -3.6093321e-13f }, { 2.1850398e+00f, -2.0841725e-12f }, 2.5e-01f },
        /* 64 */ { { -3.6093321e-13f, -3.6093321e-13f }, { -3.6093321e-13f, -3.6093321e-13f }, 1.0995116e+12f },
        /* 65 */ { { -6.5136393e-26f, -3.6093321e-13f }, { -6.5136393e-26f, -3.6093321e-13f }, 2.1990233e+12f },
        /* 66 */ { { -min, -3.6093321e-13f }, { -min, -3.6093321e-13f }, 2.1990233e+12f },
        /* 67 */ { { zero, -3.6093321e-13f }, { zero, -3.6093321e-13f }, 2.1990233e+12f },
        /* 68 */ { { min, -3.6093321e-13f }, { min, -3.6093321e-13f }, 2.1990233e+12f },
        /* 69 */ { { 6.5136393e-26f, -3.6093321e-13f }, { 6.5136393e-26f, -3.6093321e-13f }, 2.1990233e+12f },
        /* 70 */ { { 3.6093321e-13f, -3.6093321e-13f }, { 3.6093321e-13f, -3.6093321e-13f }, 1.0995116e+12f },
        /* 71 */ { { 2.e+00f, -3.6093321e-13f }, { -2.1850398e+00f, -2.0841725e-12f }, 2.5e-01f },
        /* 72 */ { { 1.1082383e+13f, -3.6093321e-13f }, { 7.0485216e-01f, -5.4025084e-13f }, 1.e+00f },
        /* 73 */ { { 6.1409603e+25f, -3.6093321e-13f }, { -1.406664e+01f, -7.1778909e-11f }, 6.25e-02f },
        /* 74 */ { { max, -3.6093321e-13f }, { -6.1179793e-01f, -4.9602932e-13f }, 1.e+00f },
        // { { inf, -3.6093321e-13f }, { nan, nan }, 1.e+00f },
        // { { -inf, -6.5136393e-26f }, { nan, nan }, 1.e+00f },
        /* 75 */ { { -max, -6.5136393e-26f }, { 6.1179793e-01f, -8.9516731e-26f }, 1.e+00f },
        /* 76 */ { { -6.1409603e+25f, -6.5136393e-26f }, { 1.406664e+01f, -1.2953697e-23f }, 6.25e-02f },
        /* 77 */ { { -1.1082383e+13f, -6.5136393e-26f }, { -7.0485216e-01f, -9.7497236e-26f }, 1.e+00f },
        /* 78 */ { { -2.e+00f, -6.5136393e-26f }, { 2.1850398e+00f, -3.7612353e-25f }, 2.5e-01f },
        /* 79 */ { { -3.6093321e-13f, -6.5136393e-26f }, { -3.6093321e-13f, -6.5136393e-26f }, 2.1990233e+12f },
        /* 80 */ { { -6.5136393e-26f, -6.5136393e-26f }, { -6.5136393e-26f, -6.5136393e-26f }, 9.6714066e+24f },
        /* 81 */ { { -min, -6.5136393e-26f }, { -min, -6.5136393e-26f }, 9.6714066e+24f },
        /* 82 */ { { zero, -6.5136393e-26f }, { zero, -6.5136393e-26f }, 9.6714066e+24f },
        /* 83 */ { { min, -6.5136393e-26f }, { min, -6.5136393e-26f }, 9.6714066e+24f },
        /* 84 */ { { 6.5136393e-26f, -6.5136393e-26f }, { 6.5136393e-26f, -6.5136393e-26f }, 9.6714066e+24f },
        /* 85 */ { { 3.6093321e-13f, -6.5136393e-26f }, { 3.6093321e-13f, -6.5136393e-26f }, 2.1990233e+12f },
        /* 86 */ { { 2.e+00f, -6.5136393e-26f }, { -2.1850398e+00f, -3.7612353e-25f }, 2.5e-01f },
        /* 87 */ { { 1.1082383e+13f, -6.5136393e-26f }, { 7.0485216e-01f, -9.7497236e-26f }, 1.e+00f },
        /* 88 */ { { 6.1409603e+25f, -6.5136393e-26f }, { -1.406664e+01f, -1.2953697e-23f }, 6.25e-02f },
        /* 89 */ { { max, -6.5136393e-26f }, { -6.1179793e-01f, -8.9516731e-26f }, 1.e+00f },
        // { { inf, -6.5136393e-26f }, { nan, nan }, 1.e+00f },
        // { { -inf, -min }, { nan, nan }, 1.e+00f },
        /* 90 */ { { -max, -min }, { 6.1179793e-01f, -1.6154781e-38f }, 1.e+00f },
        /* 91 */ { { -6.1409603e+25f, -min }, { 1.406664e+01f, -2.3377097e-36f }, 6.25e-02f },
        /* 92 */ { { -1.1082383e+13f, -min }, { -7.0485216e-01f, -1.7594995e-38f }, 1.e+00f },
        /* 93 */ { { -2.e+00f, -min }, { 2.1850398e+00f, -6.7877737e-38f }, 2.5e-01f },
        /* 94 */ { { -3.6093321e-13f, -min }, { -3.6093321e-13f, -min }, 2.1990233e+12f },
        /* 95 */ { { -6.5136393e-26f, -min }, { -6.5136393e-26f, -min }, 9.6714066e+24f },
        /* 96 */ { { -min, -min }, { -min, -min }, 4.2535296e+37f },
        /* 97 */ { { zero, -min }, { zero, -min }, 4.2535296e+37f },
        /* 98 */ { { min, -min }, { min, -min }, 4.2535296e+37f },
        /* 99 */ { { 6.5136393e-26f, -min }, { 6.5136393e-26f, -min }, 9.6714066e+24f },
        /* 100 */ { { 3.6093321e-13f, -min }, { 3.6093321e-13f, -min }, 2.1990233e+12f },
        /* 101 */ { { 2.e+00f, -min }, { -2.1850398e+00f, -6.7877737e-38f }, 2.5e-01f },
        /* 102 */ { { 1.1082383e+13f, -min }, { 7.0485216e-01f, -1.7594995e-38f }, 1.e+00f },
        /* 103 */ { { 6.1409603e+25f, -min }, { -1.406664e+01f, -2.3377097e-36f }, 6.25e-02f },
        /* 104 */ { { max, -min }, { -6.1179793e-01f, -1.6154781e-38f }, 1.e+00f },
        // { { inf, -min }, { nan, nan }, 1.e+00f },
        // { { -inf, zero }, { nan, nan }, 1.e+00f },
        /* 105 */ { { -max, zero }, { 6.1179793e-01f, zero }, 1.e+00f },
        /* 106 */ { { -6.1409603e+25f, zero }, { 1.406664e+01f, zero }, 6.25e-02f },
        /* 107 */ { { -1.1082383e+13f, zero }, { -7.0485216e-01f, zero }, 1.e+00f },
        /* 108 */ { { -2.e+00f, zero }, { 2.1850398e+00f, zero }, 2.5e-01f },
        /* 109 */ { { -3.6093321e-13f, zero }, { -3.6093321e-13f, zero }, 2.1990233e+12f },
        /* 110 */ { { -6.5136393e-26f, zero }, { -6.5136393e-26f, zero }, 9.6714066e+24f },
        /* 111 */ { { -min, zero }, { -min, zero }, 4.2535296e+37f },
        /* 112 */ { { zero, zero }, { zero, zero }, 1.e+00f },
        /* 113 */ { { min, zero }, { min, zero }, 4.2535296e+37f },
        /* 114 */ { { 6.5136393e-26f, zero }, { 6.5136393e-26f, zero }, 9.6714066e+24f },
        /* 115 */ { { 3.6093321e-13f, zero }, { 3.6093321e-13f, zero }, 2.1990233e+12f },
        /* 116 */ { { 2.e+00f, zero }, { -2.1850398e+00f, zero }, 2.5e-01f },
        /* 117 */ { { 1.1082383e+13f, zero }, { 7.0485216e-01f, zero }, 1.e+00f },
        /* 118 */ { { 6.1409603e+25f, zero }, { -1.406664e+01f, zero }, 6.25e-02f },
        /* 119 */ { { max, zero }, { -6.1179793e-01f, zero }, 1.e+00f },
        // { { inf, zero }, { nan, nan }, 1.e+00f },
        // { { -inf, min }, { nan, nan }, 1.e+00f },
        /* 120 */ { { -max, min }, { 6.1179793e-01f, 1.6154781e-38f }, 1.e+00f },
        /* 121 */ { { -6.1409603e+25f, min }, { 1.406664e+01f, 2.3377097e-36f }, 6.25e-02f },
        /* 122 */ { { -1.1082383e+13f, min }, { -7.0485216e-01f, 1.7594995e-38f }, 1.e+00f },
        /* 123 */ { { -2.e+00f, min }, { 2.1850398e+00f, 6.7877737e-38f }, 2.5e-01f },
        /* 124 */ { { -3.6093321e-13f, min }, { -3.6093321e-13f, min }, 2.1990233e+12f },
        /* 125 */ { { -6.5136393e-26f, min }, { -6.5136393e-26f, min }, 9.6714066e+24f },
        /* 126 */ { { -min, min }, { -min, min }, 4.2535296e+37f },
        /* 127 */ { { zero, min }, { zero, min }, 4.2535296e+37f },
        /* 128 */ { { min, min }, { min, min }, 4.2535296e+37f },
        /* 129 */ { { 6.5136393e-26f, min }, { 6.5136393e-26f, min }, 9.6714066e+24f },
        /* 130 */ { { 3.6093321e-13f, min }, { 3.6093321e-13f, min }, 2.1990233e+12f },
        /* 131 */ { { 2.e+00f, min }, { -2.1850398e+00f, 6.7877737e-38f }, 2.5e-01f },
        /* 132 */ { { 1.1082383e+13f, min }, { 7.0485216e-01f, 1.7594995e-38f }, 1.e+00f },
        /* 133 */ { { 6.1409603e+25f, min }, { -1.406664e+01f, 2.3377097e-36f }, 6.25e-02f },
        /* 134 */ { { max, min }, { -6.1179793e-01f, 1.6154781e-38f }, 1.e+00f },
        // { { inf, min }, { nan, nan }, 1.e+00f },
        // { { -inf, 6.5136393e-26f }, { nan, nan }, 1.e+00f },
        /* 135 */ { { -max, 6.5136393e-26f }, { 6.1179793e-01f, 8.9516731e-26f }, 1.e+00f },
        /* 136 */ { { -6.1409603e+25f, 6.5136393e-26f }, { 1.406664e+01f, 1.2953697e-23f }, 6.25e-02f },
        /* 137 */ { { -1.1082383e+13f, 6.5136393e-26f }, { -7.0485216e-01f, 9.7497236e-26f }, 1.e+00f },
        /* 138 */ { { -2.e+00f, 6.5136393e-26f }, { 2.1850398e+00f, 3.7612353e-25f }, 2.5e-01f },
        /* 139 */ { { -3.6093321e-13f, 6.5136393e-26f }, { -3.6093321e-13f, 6.5136393e-26f }, 2.1990233e+12f },
        /* 140 */ { { -6.5136393e-26f, 6.5136393e-26f }, { -6.5136393e-26f, 6.5136393e-26f }, 9.6714066e+24f },
        /* 141 */ { { -min, 6.5136393e-26f }, { -min, 6.5136393e-26f }, 9.6714066e+24f },
        /* 142 */ { { zero, 6.5136393e-26f }, { zero, 6.5136393e-26f }, 9.6714066e+24f },
        /* 143 */ { { min, 6.5136393e-26f }, { min, 6.5136393e-26f }, 9.6714066e+24f },
        /* 144 */ { { 6.5136393e-26f, 6.5136393e-26f }, { 6.5136393e-26f, 6.5136393e-26f }, 9.6714066e+24f },
        /* 145 */ { { 3.6093321e-13f, 6.5136393e-26f }, { 3.6093321e-13f, 6.5136393e-26f }, 2.1990233e+12f },
        /* 146 */ { { 2.e+00f, 6.5136393e-26f }, { -2.1850398e+00f, 3.7612353e-25f }, 2.5e-01f },
        /* 147 */ { { 1.1082383e+13f, 6.5136393e-26f }, { 7.0485216e-01f, 9.7497236e-26f }, 1.e+00f },
        /* 148 */ { { 6.1409603e+25f, 6.5136393e-26f }, { -1.406664e+01f, 1.2953697e-23f }, 6.25e-02f },
        /* 149 */ { { max, 6.5136393e-26f }, { -6.1179793e-01f, 8.9516731e-26f }, 1.e+00f },
        // { { inf, 6.5136393e-26f }, { nan, nan }, 1.e+00f },
        // { { -inf, 3.6093321e-13f }, { nan, nan }, 1.e+00f },
        /* 150 */ { { -max, 3.6093321e-13f }, { 6.1179793e-01f, 4.9602932e-13f }, 1.e+00f },
        /* 151 */ { { -6.1409603e+25f, 3.6093321e-13f }, { 1.406664e+01f, 7.1778909e-11f }, 6.25e-02f },
        /* 152 */ { { -1.1082383e+13f, 3.6093321e-13f }, { -7.0485216e-01f, 5.4025084e-13f }, 1.e+00f },
        /* 153 */ { { -2.e+00f, 3.6093321e-13f }, { 2.1850398e+00f, 2.0841725e-12f }, 2.5e-01f },
        /* 154 */ { { -3.6093321e-13f, 3.6093321e-13f }, { -3.6093321e-13f, 3.6093321e-13f }, 1.0995116e+12f },
        /* 155 */ { { -6.5136393e-26f, 3.6093321e-13f }, { -6.5136393e-26f, 3.6093321e-13f }, 2.1990233e+12f },
        /* 156 */ { { -min, 3.6093321e-13f }, { -min, 3.6093321e-13f }, 2.1990233e+12f },
        /* 157 */ { { zero, 3.6093321e-13f }, { zero, 3.6093321e-13f }, 2.1990233e+12f },
        /* 158 */ { { min, 3.6093321e-13f }, { min, 3.6093321e-13f }, 2.1990233e+12f },
        /* 159 */ { { 6.5136393e-26f, 3.6093321e-13f }, { 6.5136393e-26f, 3.6093321e-13f }, 2.1990233e+12f },
        /* 160 */ { { 3.6093321e-13f, 3.6093321e-13f }, { 3.6093321e-13f, 3.6093321e-13f }, 1.0995116e+12f },
        /* 161 */ { { 2.e+00f, 3.6093321e-13f }, { -2.1850398e+00f, 2.0841725e-12f }, 2.5e-01f },
        /* 162 */ { { 1.1082383e+13f, 3.6093321e-13f }, { 7.0485216e-01f, 5.4025084e-13f }, 1.e+00f },
        /* 163 */ { { 6.1409603e+25f, 3.6093321e-13f }, { -1.406664e+01f, 7.1778909e-11f }, 6.25e-02f },
        /* 164 */ { { max, 3.6093321e-13f }, { -6.1179793e-01f, 4.9602932e-13f }, 1.e+00f },
        // { { inf, 3.6093321e-13f }, { nan, nan }, 1.e+00f },
        // { { -inf, 2.e+00f }, { nan, nan }, 1.e+00f },
        /* 165 */ { { -max, 2.e+00f }, { 3.2068815e-02f, 9.8294145e-01f }, 1.e+00f },
        /* 166 */ { { -6.1409603e+25f, 2.e+00f }, { 5.3751757e-03f, 1.0369183e+00f }, 5.e-01f },
        /* 167 */ { { -1.1082383e+13f, 2.e+00f }, { -3.4068439e-02f, 9.8717695e-01f }, 1.e+00f },
        /* 168 */ { { -2.e+00f, 2.e+00f }, { 2.8392954e-02f, 1.0238355e+00f }, 5.e-01f },
        /* 169 */ { { -3.6093321e-13f, 2.e+00f }, { -2.5500228e-14f, 9.6402758e-01f }, 1.e+00f },
        /* 170 */ { { -6.5136393e-26f, 2.e+00f }, { -4.6019399e-27f, 9.6402758e-01f }, 1.e+00f },
        /* 171 */ { { -min, 2.e+00f }, { zero, 9.6402758e-01f }, 1.e+00f },
        /* 172 */ { { zero, 2.e+00f }, { zero, 9.6402758e-01f }, 1.e+00f },
        /* 173 */ { { min, 2.e+00f }, { zero, 9.6402758e-01f }, 1.e+00f },
        /* 174 */ { { 6.5136393e-26f, 2.e+00f }, { 4.6019399e-27f, 9.6402758e-01f }, 1.e+00f },
        /* 175 */ { { 3.6093321e-13f, 2.e+00f }, { 2.5500228e-14f, 9.6402758e-01f }, 1.e+00f },
        /* 176 */ { { 2.e+00f, 2.e+00f }, { -2.8392954e-02f, 1.0238355e+00f }, 5.e-01f },
        /* 177 */ { { 1.1082383e+13f, 2.e+00f }, { 3.4068439e-02f, 9.8717695e-01f }, 1.e+00f },
        /* 178 */ { { 6.1409603e+25f, 2.e+00f }, { -5.3751757e-03f, 1.0369183e+00f }, 5.e-01f },
        /* 179 */ { { max, 2.e+00f }, { -3.2068815e-02f, 9.8294145e-01f }, 1.e+00f },
        // { { inf, 2.e+00f }, { nan, nan }, 1.e+00f },
        // { { -inf, 1.1082383e+13f }, { nan, nan }, 1.e+00f },
        /* 180 */ { { -max, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 181 */ { { -6.1409603e+25f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 182 */ { { -1.1082383e+13f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 183 */ { { -2.e+00f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 184 */ { { -3.6093321e-13f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 185 */ { { -6.5136393e-26f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 186 */ { { -min, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 187 */ { { zero, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 188 */ { { min, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 189 */ { { 6.5136393e-26f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 190 */ { { 3.6093321e-13f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 191 */ { { 2.e+00f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 192 */ { { 1.1082383e+13f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 193 */ { { 6.1409603e+25f, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        /* 194 */ { { max, 1.1082383e+13f }, { zero, 1.e+00f }, 5.e-01f },
        // { { inf, 1.1082383e+13f }, { nan, nan }, 1.e+00f },
        // { { -inf, 6.1409603e+25f }, { nan, nan }, 1.e+00f },
        /* 195 */ { { -max, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 196 */ { { -6.1409603e+25f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 197 */ { { -1.1082383e+13f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 198 */ { { -2.e+00f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 199 */ { { -3.6093321e-13f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 200 */ { { -6.5136393e-26f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 201 */ { { -min, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 202 */ { { zero, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 203 */ { { min, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 204 */ { { 6.5136393e-26f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 205 */ { { 3.6093321e-13f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 206 */ { { 2.e+00f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 207 */ { { 1.1082383e+13f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 208 */ { { 6.1409603e+25f, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        /* 209 */ { { max, 6.1409603e+25f }, { zero, 1.e+00f }, 5.e-01f },
        // { { inf, 6.1409603e+25f }, { nan, nan }, 1.e+00f },
        // { { -inf, max }, { nan, nan }, 1.e+00f },
        /* 210 */ { { -max, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 211 */ { { -6.1409603e+25f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 212 */ { { -1.1082383e+13f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 213 */ { { -2.e+00f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 214 */ { { -3.6093321e-13f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 215 */ { { -6.5136393e-26f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 216 */ { { -min, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 217 */ { { zero, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 218 */ { { min, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 219 */ { { 6.5136393e-26f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 220 */ { { 3.6093321e-13f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 221 */ { { 2.e+00f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 222 */ { { 1.1082383e+13f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 223 */ { { 6.1409603e+25f, max }, { zero, 1.e+00f }, 5.e-01f },
        /* 224 */ { { max, max }, { zero, 1.e+00f }, 5.e-01f },
        // { { inf, max }, { nan, nan }, 1.e+00f },
        // { { -inf, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { -max, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { -6.1409603e+25f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { -1.1082383e+13f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { -2.e+00f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { -3.6093321e-13f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { -6.5136393e-26f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { -min, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { zero, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { min, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { 6.5136393e-26f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { 3.6093321e-13f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { 2.e+00f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { 1.1082383e+13f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { 6.1409603e+25f, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { max, inf }, { zero, 1.e+00f }, 5.e-01f },
        // { { inf, inf }, { zero, 1.e+00f }, 5.e-01f }
          // clang-format on
      };
      return table;
    } else if constexpr (std::is_same_v<T, double>) {
      const T zero = 0.0;
      const T min = std::numeric_limits<T>::min();
      const T max = std::numeric_limits<T>::max();
      const TableType table{
          // clang-format off
          // Ignore max 80 character line width style requirement for
          // (i) the readability
          // (ii) the consistency with the local conventions
        // { { -inf, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { -max, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { -4.0131652080900752e+205, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { -8.9589789687104559e+102, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { -1.9999999999998694e+00, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { -4.4647944971961829e-103, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { -9.9671949510973086e-206, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { -min, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { zero, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { min, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { 9.9671949510973086e-206, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { 4.4647944971961829e-103, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { 1.9999999999998694e+00, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { 8.9589789687104559e+102, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { 4.0131652080900752e+205, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { max, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { inf, -inf }, { zero, -1.e+00 }, 5.e-01 },
        // { { -inf, -max }, { nan, nan }, 1.e+00 },
        /* 0 */ { { -max, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 1 */ { { -4.0131652080900752e+205, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 2 */ { { -8.9589789687104559e+102, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 3 */ { { -1.9999999999998694e+00, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 4 */ { { -4.4647944971961829e-103, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 5 */ { { -9.9671949510973086e-206, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 6 */ { { -min, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 7 */ { { zero, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 8 */ { { min, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 9 */ { { 9.9671949510973086e-206, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 10 */ { { 4.4647944971961829e-103, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 11 */ { { 1.9999999999998694e+00, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 12 */ { { 8.9589789687104559e+102, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 13 */ { { 4.0131652080900752e+205, -max }, { zero, -1.e+00 }, 5.e-01 },
        /* 14 */ { { max, -max }, { zero, -1.e+00 }, 5.e-01 },
        // { { inf, -max }, { nan, nan }, 1.e+00 },
        // { { -inf, -4.0131652080900752e+205 }, { nan, nan }, 1.e+00 },
        /* 15 */ { { -max, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 16 */ { { -4.0131652080900752e+205, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 17 */ { { -8.9589789687104559e+102, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 18 */ { { -1.9999999999998694e+00, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 19 */ { { -4.4647944971961829e-103, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 20 */ { { -9.9671949510973086e-206, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 21 */ { { -min, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 22 */ { { zero, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 23 */ { { min, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 24 */ { { 9.9671949510973086e-206, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 25 */ { { 4.4647944971961829e-103, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 26 */ { { 1.9999999999998694e+00, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 27 */ { { 8.9589789687104559e+102, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 28 */ { { 4.0131652080900752e+205, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        /* 29 */ { { max, -4.0131652080900752e+205 }, { zero, -1.e+00 }, 5.e-01 },
        // { { inf, -4.0131652080900752e+205 }, { nan, nan }, 1.e+00 },
        // { { -inf, -8.9589789687104559e+102 }, { nan, nan }, 1.e+00 },
        /* 30 */ { { -max, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 31 */ { { -4.0131652080900752e+205, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 32 */ { { -8.9589789687104559e+102, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 33 */ { { -1.9999999999998694e+00, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 34 */ { { -4.4647944971961829e-103, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 35 */ { { -9.9671949510973086e-206, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 36 */ { { -min, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 37 */ { { zero, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 38 */ { { min, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 39 */ { { 9.9671949510973086e-206, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 40 */ { { 4.4647944971961829e-103, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 41 */ { { 1.9999999999998694e+00, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 42 */ { { 8.9589789687104559e+102, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 43 */ { { 4.0131652080900752e+205, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        /* 44 */ { { max, -8.9589789687104559e+102 }, { zero, -1.e+00 }, 5.e-01 },
        // { { inf, -8.9589789687104559e+102 }, { nan, nan }, 1.e+00 },
        // { { -inf, -1.9999999999998694e+00 }, { nan, nan }, 1.e+00 },
        /* 45 */ { { -max, -1.9999999999998694e+00 }, { 3.5056249287033966e-04, -9.6402925699855813e-01 }, 1.e+00 },
        /* 46 */ { { -4.0131652080900752e+205, -1.9999999999998694e+00 }, { 2.7597961351536058e-02, -1.02477724455786e+00 }, 5.e-01 },
        /* 47 */ { { -8.9589789687104559e+102, -1.9999999999998694e+00 }, { 3.6482534190112638e-02, -1.0041027601252124e+00 }, 5.e-01 },
        /* 48 */ { { -1.9999999999998694e+00, -1.9999999999998694e+00 }, { 2.8392952868233685e-02, -1.0238355945704865e+00 }, 5.e-01 },
        /* 49 */ { { -4.4647944971961829e-103, -1.9999999999998694e+00 }, { -3.1544141402685944e-104, -9.6402758007580769e-01 }, 1.e+00 },
        /* 50 */ { { -9.9671949510973086e-206, -1.9999999999998694e+00 }, { -7.0419054476749839e-207, -9.6402758007580769e-01 }, 1.e+00 },
        /* 51 */ { { -min, -1.9999999999998694e+00 }, { zero, -9.6402758007580769e-01 }, 1.e+00 },
        /* 52 */ { { zero, -1.9999999999998694e+00 }, { zero, -9.6402758007580769e-01 }, 1.e+00 },
        /* 53 */ { { min, -1.9999999999998694e+00 }, { zero, -9.6402758007580769e-01 }, 1.e+00 },
        /* 54 */ { { 9.9671949510973086e-206, -1.9999999999998694e+00 }, { 7.0419054476749839e-207, -9.6402758007580769e-01 }, 1.e+00 },
        /* 55 */ { { 4.4647944971961829e-103, -1.9999999999998694e+00 }, { 3.1544141402685944e-104, -9.6402758007580769e-01 }, 1.e+00 },
        /* 56 */ { { 1.9999999999998694e+00, -1.9999999999998694e+00 }, { -2.8392952868233685e-02, -1.0238355945704865e+00 }, 5.e-01 },
        /* 57 */ { { 8.9589789687104559e+102, -1.9999999999998694e+00 }, { -3.6482534190112638e-02, -1.0041027601252124e+00 }, 5.e-01 },
        /* 58 */ { { 4.0131652080900752e+205, -1.9999999999998694e+00 }, { -2.7597961351536058e-02, -1.02477724455786e+00 }, 5.e-01 },
        /* 59 */ { { max, -1.9999999999998694e+00 }, { -3.5056249287033966e-04, -9.6402925699855813e-01 }, 1.e+00 },
        // { { inf, -1.9999999999998694e+00 }, { nan, nan }, 1.e+00 },
        // { { -inf, -4.4647944971961829e-103 }, { nan, nan }, 1.e+00 },
        /* 60 */ { { -max, -4.4647944971961829e-103 }, { 4.9620158744448951e-03, -4.4649044275872426e-103 }, 1.28e+02 },
        /* 61 */ { { -4.0131652080900752e+205, -4.4647944971961829e-103 }, { 2.2833759509915996e+00, -2.7743365645875101e-102 }, 2.5e-01 },
        /* 62 */ { { -8.9589789687104559e+102, -4.4647944971961829e-103 }, { 1.1394650926503123e+00, -1.0261797490073182e-102 }, 5.e-01 },
        /* 63 */ { { -1.9999999999998694e+00, -4.4647944971961829e-103 }, { 2.185039863262273e+00, -2.5781505790835082e-102 }, 2.5e-01 },
        /* 64 */ { { -4.4647944971961829e-103, -4.4647944971961829e-103 }, { -4.4647944971961829e-103, -4.4647944971961829e-103 }, 1.1198723710889021e+102 },
        /* 65 */ { { -9.9671949510973086e-206, -4.4647944971961829e-103 }, { -9.9671949510973086e-206, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 66 */ { { -min, -4.4647944971961829e-103 }, { -min, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 67 */ { { zero, -4.4647944971961829e-103 }, { zero, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 68 */ { { min, -4.4647944971961829e-103 }, { min, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 69 */ { { 9.9671949510973086e-206, -4.4647944971961829e-103 }, { 9.9671949510973086e-206, -4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 70 */ { { 4.4647944971961829e-103, -4.4647944971961829e-103 }, { 4.4647944971961829e-103, -4.4647944971961829e-103 }, 1.1198723710889021e+102 },
        /* 71 */ { { 1.9999999999998694e+00, -4.4647944971961829e-103 }, { -2.185039863262273e+00, -2.5781505790835082e-102 }, 2.5e-01 },
        /* 72 */ { { 8.9589789687104559e+102, -4.4647944971961829e-103 }, { -1.1394650926503123e+00, -1.0261797490073182e-102 }, 5.e-01 },
        /* 73 */ { { 4.0131652080900752e+205, -4.4647944971961829e-103 }, { -2.2833759509915996e+00, -2.7743365645875101e-102 }, 2.5e-01 },
        /* 74 */ { { max, -4.4647944971961829e-103 }, { -4.9620158744448951e-03, -4.4649044275872426e-103 }, 1.28e+02 },
        // { { inf, -4.4647944971961829e-103 }, { nan, nan }, 1.e+00 },
        // { { -inf, -9.9671949510973086e-206 }, { nan, nan }, 1.e+00 },
        /* 75 */ { { -max, -9.9671949510973086e-206 }, { 4.9620158744448951e-03, -9.9674403593998487e-206 }, 1.28e+02 },
        /* 76 */ { { -4.0131652080900752e+205, -9.9671949510973086e-206 }, { 2.2833759509915996e+00, -6.1934213134706426e-205 }, 2.5e-01 },
        /* 77 */ { { -8.9589789687104559e+102, -9.9671949510973086e-206 }, { 1.1394650926503123e+00, -2.2908408482511669e-205 }, 5.e-01 },
        /* 78 */ { { -1.9999999999998694e+00, -9.9671949510973086e-206 }, { 2.185039863262273e+00, -5.7554562592179759e-205 }, 2.5e-01 },
        /* 79 */ { { -4.4647944971961829e-103, -9.9671949510973086e-206 }, { -4.4647944971961829e-103, -9.9671949510973086e-206 }, 2.2397447421778042e+102 },
        /* 80 */ { { -9.9671949510973086e-206, -9.9671949510973086e-206 }, { -9.9671949510973086e-206, -9.9671949510973086e-206 }, 5.0164565101131187e+204 },
        /* 81 */ { { -min, -9.9671949510973086e-206 }, { -min, -9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 82 */ { { zero, -9.9671949510973086e-206 }, { zero, -9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 83 */ { { min, -9.9671949510973086e-206 }, { min, -9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 84 */ { { 9.9671949510973086e-206, -9.9671949510973086e-206 }, { 9.9671949510973086e-206, -9.9671949510973086e-206 }, 5.0164565101131187e+204 },
        /* 85 */ { { 4.4647944971961829e-103, -9.9671949510973086e-206 }, { 4.4647944971961829e-103, -9.9671949510973086e-206 }, 2.2397447421778042e+102 },
        /* 86 */ { { 1.9999999999998694e+00, -9.9671949510973086e-206 }, { -2.185039863262273e+00, -5.7554562592179759e-205 }, 2.5e-01 },
        /* 87 */ { { 8.9589789687104559e+102, -9.9671949510973086e-206 }, { -1.1394650926503123e+00, -2.2908408482511669e-205 }, 5.e-01 },
        /* 88 */ { { 4.0131652080900752e+205, -9.9671949510973086e-206 }, { -2.2833759509915996e+00, -6.1934213134706426e-205 }, 2.5e-01 },
        /* 89 */ { { max, -9.9671949510973086e-206 }, { -4.9620158744448951e-03, -9.9674403593998487e-206 }, 1.28e+02 },
        // { { inf, -9.9671949510973086e-206 }, { nan, nan }, 1.e+00 },
        // { { -inf, -min }, { nan, nan }, 1.e+00 },
        /* 90 */ { { -max, -min }, { 4.9620158744448951e-03, -2.2251286433891388e-308 }, 1.28e+02 },
        /* 91 */ { { -4.0131652080900752e+205, -min }, { 2.2833759509915996e+00, -1.3826176699601631e-307 }, 2.5e-01 },
        /* 92 */ { { -8.9589789687104559e+102, -min }, { 1.1394650926503123e+00, -5.1140668066123887e-308 }, 5.e-01 },
        /* 93 */ { { -1.9999999999998694e+00, -min }, { 2.185039863262273e+00, -1.2848464717505794e-307 }, 2.5e-01 },
        /* 94 */ { { -4.4647944971961829e-103, -min }, { -4.4647944971961829e-103, -min }, 2.2397447421778042e+102 },
        /* 95 */ { { -9.9671949510973086e-206, -min }, { -9.9671949510973086e-206, -min }, 1.0032913020226237e+205 },
        /* 96 */ { { -min, -min }, { -min, -min }, 2.2471164185778949e+307 },
        /* 97 */ { { zero, -min }, { zero, -min }, 2.2471164185778949e+307 },
        /* 98 */ { { min, -min }, { min, -min }, 2.2471164185778949e+307 },
        /* 99 */ { { 9.9671949510973086e-206, -min }, { 9.9671949510973086e-206, -min }, 1.0032913020226237e+205 },
        /* 100 */ { { 4.4647944971961829e-103, -min }, { 4.4647944971961829e-103, -min }, 2.2397447421778042e+102 },
        /* 101 */ { { 1.9999999999998694e+00, -min }, { -2.185039863262273e+00, -1.2848464717505794e-307 }, 2.5e-01 },
        /* 102 */ { { 8.9589789687104559e+102, -min }, { -1.1394650926503123e+00, -5.1140668066123887e-308 }, 5.e-01 },
        /* 103 */ { { 4.0131652080900752e+205, -min }, { -2.2833759509915996e+00, -1.3826176699601631e-307 }, 2.5e-01 },
        /* 104 */ { { max, -min }, { -4.9620158744448951e-03, -2.2251286433891388e-308 }, 1.28e+02 },
        // { { inf, -min }, { nan, nan }, 1.e+00 },
        // { { -inf, zero }, { nan, nan }, 1.e+00 },
        /* 105 */ { { -max, zero }, { 4.9620158744448951e-03, zero }, 1.28e+02 },
        /* 106 */ { { -4.0131652080900752e+205, zero }, { 2.2833759509915996e+00, zero }, 2.5e-01 },
        /* 107 */ { { -8.9589789687104559e+102, zero }, { 1.1394650926503123e+00, zero }, 5.e-01 },
        /* 108 */ { { -1.9999999999998694e+00, zero }, { 2.185039863262273e+00, zero }, 2.5e-01 },
        /* 109 */ { { -4.4647944971961829e-103, zero }, { -4.4647944971961829e-103, zero }, 2.2397447421778042e+102 },
        /* 110 */ { { -9.9671949510973086e-206, zero }, { -9.9671949510973086e-206, zero }, 1.0032913020226237e+205 },
        /* 111 */ { { -min, zero }, { -min, zero }, 2.2471164185778949e+307 },
        /* 112 */ { { zero, zero }, { zero, zero }, 1.e+00 },
        /* 113 */ { { min, zero }, { min, zero }, 2.2471164185778949e+307 },
        /* 114 */ { { 9.9671949510973086e-206, zero }, { 9.9671949510973086e-206, zero }, 1.0032913020226237e+205 },
        /* 115 */ { { 4.4647944971961829e-103, zero }, { 4.4647944971961829e-103, zero }, 2.2397447421778042e+102 },
        /* 116 */ { { 1.9999999999998694e+00, zero }, { -2.185039863262273e+00, zero }, 2.5e-01 },
        /* 117 */ { { 8.9589789687104559e+102, zero }, { -1.1394650926503123e+00, zero }, 5.e-01 },
        /* 118 */ { { 4.0131652080900752e+205, zero }, { -2.2833759509915996e+00, zero }, 2.5e-01 },
        /* 119 */ { { max, zero }, { -4.9620158744448951e-03, zero }, 1.28e+02 },
        // { { inf, zero }, { nan, nan }, 1.e+00 },
        // { { -inf, min }, { nan, nan }, 1.e+00 },
        /* 120 */ { { -max, min }, { 4.9620158744448951e-03, 2.2251286433891388e-308 }, 1.28e+02 },
        /* 121 */ { { -4.0131652080900752e+205, min }, { 2.2833759509915996e+00, 1.3826176699601631e-307 }, 2.5e-01 },
        /* 122 */ { { -8.9589789687104559e+102, min }, { 1.1394650926503123e+00, 5.1140668066123887e-308 }, 5.e-01 },
        /* 123 */ { { -1.9999999999998694e+00, min }, { 2.185039863262273e+00, 1.2848464717505794e-307 }, 2.5e-01 },
        /* 124 */ { { -4.4647944971961829e-103, min }, { -4.4647944971961829e-103, min }, 2.2397447421778042e+102 },
        /* 125 */ { { -9.9671949510973086e-206, min }, { -9.9671949510973086e-206, min }, 1.0032913020226237e+205 },
        /* 126 */ { { -min, min }, { -min, min }, 2.2471164185778949e+307 },
        /* 127 */ { { zero, min }, { zero, min }, 2.2471164185778949e+307 },
        /* 128 */ { { min, min }, { min, min }, 2.2471164185778949e+307 },
        /* 129 */ { { 9.9671949510973086e-206, min }, { 9.9671949510973086e-206, min }, 1.0032913020226237e+205 },
        /* 130 */ { { 4.4647944971961829e-103, min }, { 4.4647944971961829e-103, min }, 2.2397447421778042e+102 },
        /* 131 */ { { 1.9999999999998694e+00, min }, { -2.185039863262273e+00, 1.2848464717505794e-307 }, 2.5e-01 },
        /* 132 */ { { 8.9589789687104559e+102, min }, { -1.1394650926503123e+00, 5.1140668066123887e-308 }, 5.e-01 },
        /* 133 */ { { 4.0131652080900752e+205, min }, { -2.2833759509915996e+00, 1.3826176699601631e-307 }, 2.5e-01 },
        /* 134 */ { { max, min }, { -4.9620158744448951e-03, 2.2251286433891388e-308 }, 1.28e+02 },
        // { { inf, min }, { nan, nan }, 1.e+00 },
        // { { -inf, 9.9671949510973086e-206 }, { nan, nan }, 1.e+00 },
        /* 135 */ { { -max, 9.9671949510973086e-206 }, { 4.9620158744448951e-03, 9.9674403593998487e-206 }, 1.28e+02 },
        /* 136 */ { { -4.0131652080900752e+205, 9.9671949510973086e-206 }, { 2.2833759509915996e+00, 6.1934213134706426e-205 }, 2.5e-01 },
        /* 137 */ { { -8.9589789687104559e+102, 9.9671949510973086e-206 }, { 1.1394650926503123e+00, 2.2908408482511669e-205 }, 5.e-01 },
        /* 138 */ { { -1.9999999999998694e+00, 9.9671949510973086e-206 }, { 2.185039863262273e+00, 5.7554562592179759e-205 }, 2.5e-01 },
        /* 139 */ { { -4.4647944971961829e-103, 9.9671949510973086e-206 }, { -4.4647944971961829e-103, 9.9671949510973086e-206 }, 2.2397447421778042e+102 },
        /* 140 */ { { -9.9671949510973086e-206, 9.9671949510973086e-206 }, { -9.9671949510973086e-206, 9.9671949510973086e-206 }, 5.0164565101131187e+204 },
        /* 141 */ { { -min, 9.9671949510973086e-206 }, { -min, 9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 142 */ { { zero, 9.9671949510973086e-206 }, { zero, 9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 143 */ { { min, 9.9671949510973086e-206 }, { min, 9.9671949510973086e-206 }, 1.0032913020226237e+205 },
        /* 144 */ { { 9.9671949510973086e-206, 9.9671949510973086e-206 }, { 9.9671949510973086e-206, 9.9671949510973086e-206 }, 5.0164565101131187e+204 },
        /* 145 */ { { 4.4647944971961829e-103, 9.9671949510973086e-206 }, { 4.4647944971961829e-103, 9.9671949510973086e-206 }, 2.2397447421778042e+102 },
        /* 146 */ { { 1.9999999999998694e+00, 9.9671949510973086e-206 }, { -2.185039863262273e+00, 5.7554562592179759e-205 }, 2.5e-01 },
        /* 147 */ { { 8.9589789687104559e+102, 9.9671949510973086e-206 }, { -1.1394650926503123e+00, 2.2908408482511669e-205 }, 5.e-01 },
        /* 148 */ { { 4.0131652080900752e+205, 9.9671949510973086e-206 }, { -2.2833759509915996e+00, 6.1934213134706426e-205 }, 2.5e-01 },
        /* 149 */ { { max, 9.9671949510973086e-206 }, { -4.9620158744448951e-03, 9.9674403593998487e-206 }, 1.28e+02 },
        // { { inf, 9.9671949510973086e-206 }, { nan, nan }, 1.e+00 },
        // { { -inf, 4.4647944971961829e-103 }, { nan, nan }, 1.e+00 },
        /* 150 */ { { -max, 4.4647944971961829e-103 }, { 4.9620158744448951e-03, 4.4649044275872426e-103 }, 1.28e+02 },
        /* 151 */ { { -4.0131652080900752e+205, 4.4647944971961829e-103 }, { 2.2833759509915996e+00, 2.7743365645875101e-102 }, 2.5e-01 },
        /* 152 */ { { -8.9589789687104559e+102, 4.4647944971961829e-103 }, { 1.1394650926503123e+00, 1.0261797490073182e-102 }, 5.e-01 },
        /* 153 */ { { -1.9999999999998694e+00, 4.4647944971961829e-103 }, { 2.185039863262273e+00, 2.5781505790835082e-102 }, 2.5e-01 },
        /* 154 */ { { -4.4647944971961829e-103, 4.4647944971961829e-103 }, { -4.4647944971961829e-103, 4.4647944971961829e-103 }, 1.1198723710889021e+102 },
        /* 155 */ { { -9.9671949510973086e-206, 4.4647944971961829e-103 }, { -9.9671949510973086e-206, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 156 */ { { -min, 4.4647944971961829e-103 }, { -min, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 157 */ { { zero, 4.4647944971961829e-103 }, { zero, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 158 */ { { min, 4.4647944971961829e-103 }, { min, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 159 */ { { 9.9671949510973086e-206, 4.4647944971961829e-103 }, { 9.9671949510973086e-206, 4.4647944971961829e-103 }, 2.2397447421778042e+102 },
        /* 160 */ { { 4.4647944971961829e-103, 4.4647944971961829e-103 }, { 4.4647944971961829e-103, 4.4647944971961829e-103 }, 1.1198723710889021e+102 },
        /* 161 */ { { 1.9999999999998694e+00, 4.4647944971961829e-103 }, { -2.185039863262273e+00, 2.5781505790835082e-102 }, 2.5e-01 },
        /* 162 */ { { 8.9589789687104559e+102, 4.4647944971961829e-103 }, { -1.1394650926503123e+00, 1.0261797490073182e-102 }, 5.e-01 },
        /* 163 */ { { 4.0131652080900752e+205, 4.4647944971961829e-103 }, { -2.2833759509915996e+00, 2.7743365645875101e-102 }, 2.5e-01 },
        /* 164 */ { { max, 4.4647944971961829e-103 }, { -4.9620158744448951e-03, 4.4649044275872426e-103 }, 1.28e+02 },
        // { { inf, 4.4647944971961829e-103 }, { nan, nan }, 1.e+00 },
        // { { -inf, 1.9999999999998694e+00 }, { nan, nan }, 1.e+00 },
        /* 165 */ { { -max, 1.9999999999998694e+00 }, { 3.5056249287033966e-04, 9.6402925699855813e-01 }, 1.e+00 },
        /* 166 */ { { -4.0131652080900752e+205, 1.9999999999998694e+00 }, { 2.7597961351536058e-02, 1.02477724455786e+00 }, 5.e-01 },
        /* 167 */ { { -8.9589789687104559e+102, 1.9999999999998694e+00 }, { 3.6482534190112638e-02, 1.0041027601252124e+00 }, 5.e-01 },
        /* 168 */ { { -1.9999999999998694e+00, 1.9999999999998694e+00 }, { 2.8392952868233685e-02, 1.0238355945704865e+00 }, 5.e-01 },
        /* 169 */ { { -4.4647944971961829e-103, 1.9999999999998694e+00 }, { -3.1544141402685944e-104, 9.6402758007580769e-01 }, 1.e+00 },
        /* 170 */ { { -9.9671949510973086e-206, 1.9999999999998694e+00 }, { -7.0419054476749839e-207, 9.6402758007580769e-01 }, 1.e+00 },
        /* 171 */ { { -min, 1.9999999999998694e+00 }, { zero, 9.6402758007580769e-01 }, 1.e+00 },
        /* 172 */ { { zero, 1.9999999999998694e+00 }, { zero, 9.6402758007580769e-01 }, 1.e+00 },
        /* 173 */ { { min, 1.9999999999998694e+00 }, { zero, 9.6402758007580769e-01 }, 1.e+00 },
        /* 174 */ { { 9.9671949510973086e-206, 1.9999999999998694e+00 }, { 7.0419054476749839e-207, 9.6402758007580769e-01 }, 1.e+00 },
        /* 175 */ { { 4.4647944971961829e-103, 1.9999999999998694e+00 }, { 3.1544141402685944e-104, 9.6402758007580769e-01 }, 1.e+00 },
        /* 176 */ { { 1.9999999999998694e+00, 1.9999999999998694e+00 }, { -2.8392952868233685e-02, 1.0238355945704865e+00 }, 5.e-01 },
        /* 177 */ { { 8.9589789687104559e+102, 1.9999999999998694e+00 }, { -3.6482534190112638e-02, 1.0041027601252124e+00 }, 5.e-01 },
        /* 178 */ { { 4.0131652080900752e+205, 1.9999999999998694e+00 }, { -2.7597961351536058e-02, 1.02477724455786e+00 }, 5.e-01 },
        /* 179 */ { { max, 1.9999999999998694e+00 }, { -3.5056249287033966e-04, 9.6402925699855813e-01 }, 1.e+00 },
        // { { inf, 1.9999999999998694e+00 }, { nan, nan }, 1.e+00 },
        // { { -inf, 8.9589789687104559e+102 }, { nan, nan }, 1.e+00 },
        /* 180 */ { { -max, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 181 */ { { -4.0131652080900752e+205, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 182 */ { { -8.9589789687104559e+102, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 183 */ { { -1.9999999999998694e+00, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 184 */ { { -4.4647944971961829e-103, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 185 */ { { -9.9671949510973086e-206, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 186 */ { { -min, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 187 */ { { zero, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 188 */ { { min, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 189 */ { { 9.9671949510973086e-206, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 190 */ { { 4.4647944971961829e-103, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 191 */ { { 1.9999999999998694e+00, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 192 */ { { 8.9589789687104559e+102, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 193 */ { { 4.0131652080900752e+205, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        /* 194 */ { { max, 8.9589789687104559e+102 }, { zero, 1.e+00 }, 5.e-01 },
        // { { inf, 8.9589789687104559e+102 }, { nan, nan }, 1.e+00 },
        // { { -inf, 4.0131652080900752e+205 }, { nan, nan }, 1.e+00 },
        /* 195 */ { { -max, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 196 */ { { -4.0131652080900752e+205, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 197 */ { { -8.9589789687104559e+102, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 198 */ { { -1.9999999999998694e+00, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 199 */ { { -4.4647944971961829e-103, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 200 */ { { -9.9671949510973086e-206, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 201 */ { { -min, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 202 */ { { zero, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 203 */ { { min, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 204 */ { { 9.9671949510973086e-206, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 205 */ { { 4.4647944971961829e-103, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 206 */ { { 1.9999999999998694e+00, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 207 */ { { 8.9589789687104559e+102, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 208 */ { { 4.0131652080900752e+205, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        /* 209 */ { { max, 4.0131652080900752e+205 }, { zero, 1.e+00 }, 5.e-01 },
        // { { inf, 4.0131652080900752e+205 }, { nan, nan }, 1.e+00 },
        // { { -inf, max }, { nan, nan }, 1.e+00 },
        /* 210 */ { { -max, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 211 */ { { -4.0131652080900752e+205, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 212 */ { { -8.9589789687104559e+102, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 213 */ { { -1.9999999999998694e+00, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 214 */ { { -4.4647944971961829e-103, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 215 */ { { -9.9671949510973086e-206, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 216 */ { { -min, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 217 */ { { zero, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 218 */ { { min, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 219 */ { { 9.9671949510973086e-206, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 220 */ { { 4.4647944971961829e-103, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 221 */ { { 1.9999999999998694e+00, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 222 */ { { 8.9589789687104559e+102, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 223 */ { { 4.0131652080900752e+205, max }, { zero, 1.e+00 }, 5.e-01 },
        /* 224 */ { { max, max }, { zero, 1.e+00 }, 5.e-01 },
        // { { inf, max }, { nan, nan }, 1.e+00 },
        // { { -inf, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { -max, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { -4.0131652080900752e+205, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { -8.9589789687104559e+102, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { -1.9999999999998694e+00, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { -4.4647944971961829e-103, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { -9.9671949510973086e-206, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { -min, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { zero, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { min, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { 9.9671949510973086e-206, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { 4.4647944971961829e-103, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { 1.9999999999998694e+00, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { 8.9589789687104559e+102, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { 4.0131652080900752e+205, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { max, inf }, { zero, 1.e+00 }, 5.e-01 },
        // { { inf, inf }, { zero, 1.e+00 }, 5.e-01 }
          // clang-format on
      };
      return table;
    } else {
      static_assert(dependent_false<T>); /* unreachable */
    }
  }
};
// NOLINTEND(whitespace/line_length)

}  // namespace complex_unary_op_samples

#endif  // XLA_TESTS_COMPLEX_UNARY_OP_SAMPLES_H_
